import doctest

from insights.parsers import qpid_stat
from insights.tests import context_wrap


QPID_STAT_Q_DOCS = '''
Queues
  queue                                                                      dur  autoDel  excl  msg   msgIn  msgOut  bytes  bytesIn  bytesOut  cons  bind
  ==========================================================================================================================================================
  00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0                                        Y        Y        0     2      2       0    486      486         1     2
  0f7f1a3d-daff-42a6-a994-29050a2eabde:1.0                                        Y        Y        0     8      8       0   4.88k    4.88k        1     2
'''

QPID_STAT_U_DOCS = '''
Subscriptions
  subscr               queue                                                                      conn                                    procName          procId  browse  acked  excl  creditMode  delivered  sessUnacked
  ===========================================================================================================================================================================================================================
  0                    00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0                                   qpid.10.20.1.10:5671-10.20.1.10:33787   celery            21409                        CREDIT      2          0
  0                    pulp.agent.c6a430bc-5ec7-42f8-99ce-f320ed0b9113                            qpid.10.20.1.10:5671-10.30.0.148:57423  goferd            32227           Y            CREDIT      0          0
  1                    server.example.com:event                                                   qpid.10.20.1.10:5671-10.20.1.10:33848   Qpid Java Client  21066           Y      Y     WINDOW      2,623      0
  0                    celeryev.4c77bd03-1cde-49eb-bdc0-b7c38f9ff93d                              qpid.10.20.1.10:5671-10.20.1.10:33777   celery            21356           Y            CREDIT      363,228    0
  1                    celery                                                                     qpid.10.20.1.10:5671-10.20.1.10:33786   celery            21409           Y            CREDIT      5          0
'''

QPID_STAT_G_DOCS = '''
Broker Summary:
  uptime          cluster       connections  sessions  exchanges  queues
  ========================================================================
  97d 0h 16m 24s  <standalone>  23           37        14         33

Aggregate Broker Statistics:
  Statistic                   Messages    Bytes
  ========================================================
  queue-depth                 0           0
  total-enqueues              1,727,129   42,597,828,309
  total-dequeues              1,727,129   42,597,828,309
  persistent-enqueues         28,733      23,896,468
  persistent-dequeues         28,733      23,896,468
  transactional-enqueues      0           0
  transactional-dequeues      0           0
  flow-to-disk-depth          0           0
  flow-to-disk-enqueues       0           0
  flow-to-disk-dequeues       0           0
  acquires                    1,727,129
  releases                    0
  discards-no-route           41,171,496
  discards-ttl-expired        0
  discards-limit-overflow     0
  discards-ring-overflow      0
  discards-lvq-replace        0
  discards-subscriber-reject  0
  discards-purged             0
  reroutes                    0
  abandoned                   0
  abandoned-via-alt           0
'''


def test_qpid_stat_q_docs():
    env = {
        'qpid_stat_g': qpid_stat.QpidStatG(context_wrap(QPID_STAT_G_DOCS)),
        'qpid_stat_q': qpid_stat.QpidStatQ(context_wrap(QPID_STAT_Q_DOCS)),
        'qpid_stat_u': qpid_stat.QpidStatU(context_wrap(QPID_STAT_U_DOCS))
    }
    failed, total = doctest.testmod(qpid_stat, globs=env)
    assert failed == 0


QPID_STAT_Q = """
COMMAND> qpid-stat -q --ssl-certificate=/etc/pki/katello/qpid_client_striped.crt -b amqps://localhost:5671

Queues
  queue                                                                      dur  autoDel  excl  msg   msgIn  msgOut  bytes  bytesIn  bytesOut  cons  bind
  ==========================================================================================================================================================
  00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0                                        Y        Y        0     2      2       0    486      486         1     2
  server.example.com:event                                                   Y             Y        0  2.62k  2.62k      0   45.5m    45.5m        1     2
  celery                                                                     Y                      4    41     37    4.12k  37.5k    33.4k        8     2
  pulp.agent.836a7366-4790-482d-b3bc-efee9d42b3cd                            Y                      1     1      0     463    463        0         0     1
  reserved_resource_worker-7@server.example.com.celery.pidbox                     Y                 0     0      0       0      0        0         1     2
  reserved_resource_worker-7@server.example.com.dq                           Y    Y                 0   182    182       0    229k     229k        1     2
""".strip()


def test_qpid_stat_q():
    qpid_list = qpid_stat.QpidStatQ(context_wrap(QPID_STAT_Q))
    assert qpid_list.data[0].get('queue') == '00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0'
    assert qpid_list.data[0].get('dur') == ''
    assert qpid_list.data[1].get('queue') == 'server.example.com:event'
    assert qpid_list.data[1].get('dur') == 'Y'
    assert qpid_list.data[1].get('autoDel') == ''
    assert qpid_list.data[1].get('excl') == 'Y'
    assert qpid_list.data[1].get('msg') == '0'
    assert qpid_list.data[1].get('msgIn') == '2.62k'
    assert qpid_list.data[1].get('msgOut') == '2.62k'
    assert qpid_list.data[1].get('bytes') == '0'
    assert qpid_list.data[1].get('bytesIn') == '45.5m'
    assert qpid_list.data[1].get('bytesOut') == '45.5m'
    assert qpid_list.data[1].get('cons') == '1'
    assert qpid_list.data[1].get('bind') == '2'
    assert qpid_list.data[2].get('msg') == '4'
    assert qpid_list.data[3].get('cons') == '0'
    assert qpid_list.data[4].get('bytesIn') == '0'
    assert qpid_list.data[5].get('queue') == 'reserved_resource_worker-7@server.example.com.dq'
    assert qpid_list.data[5].get('dur') == 'Y'
    assert qpid_list.data[5].get('autoDel') == 'Y'
    assert qpid_list.data[5].get('excl') == ''
    assert qpid_list.data[5].get('msg') == '0'
    assert qpid_list.data[5].get('msgIn') == '182'
    assert qpid_list.data[5].get('msgOut') == '182'
    assert qpid_list.data[5].get('bytes') == '0'
    assert qpid_list.data[5].get('bytesIn') == '229k'
    assert qpid_list.data[5].get('bytesOut') == '229k'
    assert qpid_list.data[5].get('cons') == '1'
    assert qpid_list.data[5].get('bind') == '2'

    # test iteration
    assert [d['queue'] for d in qpid_list] == [
        '00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0',
        'server.example.com:event',
        'celery',
        'pulp.agent.836a7366-4790-482d-b3bc-efee9d42b3cd',
        'reserved_resource_worker-7@server.example.com.celery.pidbox',
        'reserved_resource_worker-7@server.example.com.dq',
    ]


QPID_STAT_U = """
COMMAND> qpid-stat -u --ssl-certificate=/etc/pki/katello/qpid_client_striped.crt -b amqps://localhost:5671

Subscriptions
  subscr               queue                                                                      conn                                    procName          procId  browse  acked  excl  creditMode  delivered  sessUnacked
  ===========================================================================================================================================================================================================================
  0                    00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0                                   qpid.10.20.1.10:5671-10.20.1.10:33787   celery            21409                        CREDIT      2          0
  0                    pulp.agent.c6a430bc-5ec7-42f8-99ce-f320ed0b9113                            qpid.10.20.1.10:5671-10.30.0.148:57423  goferd            32227           Y            CREDIT      0          0
  1                    server.example.com:event                                                   qpid.10.20.1.10:5671-10.20.1.10:33848   Qpid Java Client  21066           Y      Y     WINDOW      2,623      0
  0                    celeryev.4c77bd03-1cde-49eb-bdc0-b7c38f9ff93d                              qpid.10.20.1.10:5671-10.20.1.10:33777   celery            21356           Y            CREDIT      363,228    0
  1                    celery                                                                     qpid.10.20.1.10:5671-10.20.1.10:33786   celery            21409           Y            CREDIT      5          0
  katello_event_queue  katello_event_queue                                                        qpid.10.20.1.10:5671-10.20.1.10:33911   ruby              21801           Y            CREDIT      7,642      0
""".strip()


def test_qpid_stat_u():
    qpid_list = qpid_stat.QpidStatU(context_wrap(QPID_STAT_U))
    assert qpid_list.data[0].get('subscr') == '0'
    assert qpid_list.data[0].get('queue') == '00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0'
    assert qpid_list.data[0].get('conn') == 'qpid.10.20.1.10:5671-10.20.1.10:33787'
    assert qpid_list.data[0].get('procName') == 'celery'
    assert qpid_list.data[0].get('procId') == '21409'
    assert qpid_list.data[0].get('browse') == ''
    assert qpid_list.data[0].get('acked') == ''
    assert qpid_list.data[0].get('excl') == ''
    assert qpid_list.data[0].get('creditMode') == 'CREDIT'
    assert qpid_list.data[0].get('delivered') == '2'
    assert qpid_list.data[0].get('sessUnacked') == '0'
    assert qpid_list.data[1].get('queue') == 'pulp.agent.c6a430bc-5ec7-42f8-99ce-f320ed0b9113'
    assert qpid_list.data[1].get('conn') == 'qpid.10.20.1.10:5671-10.30.0.148:57423'
    assert qpid_list.data[1].get('acked') == 'Y'
    assert qpid_list.data[1].get('procName') == 'goferd'
    assert qpid_list.data[2].get('subscr') == '1'
    assert qpid_list.data[2].get('queue') == 'server.example.com:event'
    assert qpid_list.data[2].get('conn') == 'qpid.10.20.1.10:5671-10.20.1.10:33848'
    assert qpid_list.data[2].get('procName') == 'Qpid Java Client'
    assert qpid_list.data[2].get('procId') == '21066'
    assert qpid_list.data[2].get('browse') == ''
    assert qpid_list.data[2].get('acked') == 'Y'
    assert qpid_list.data[2].get('excl') == 'Y'
    assert qpid_list.data[2].get('creditMode') == 'WINDOW'
    assert qpid_list.data[2].get('delivered') == '2,623'
    assert qpid_list.data[2].get('sessUnacked') == '0'
    assert qpid_list.data[3].get('delivered') == '363,228'
    assert qpid_list.data[5].get('subscr') == 'katello_event_queue'

    # test iteration
    assert [d['queue'] for d in qpid_list] == [
        '00d6cc19-15fc-4b7c-af3c-6a38e7bb386d:1.0',
        'pulp.agent.c6a430bc-5ec7-42f8-99ce-f320ed0b9113',
        'server.example.com:event',
        'celeryev.4c77bd03-1cde-49eb-bdc0-b7c38f9ff93d',
        'celery',
        'katello_event_queue',
    ]


QPID_STAT_G = """
COMMAND> qpid-stat -g --ssl-certificate=/etc/pki/katello/qpid_client_striped.crt -b amqps://localhost:5671

Broker Summary:
  uptime          cluster       connections  sessions  exchanges  queues
  ========================================================================
  97d 0h 16m 24s  <standalone>  23           37        14         33

Aggregate Broker Statistics:
  Statistic                   Messages    Bytes
  ========================================================
  queue-depth                 0           0
  total-enqueues              1,727,129   42,597,828,309
  total-dequeues              1,727,129   42,597,828,309
  persistent-enqueues         28,733      23,896,468
  persistent-dequeues         28,733      23,896,468
  transactional-enqueues      0           0
  transactional-dequeues      0           0
  flow-to-disk-depth          0           0
  flow-to-disk-enqueues       0           0
  flow-to-disk-dequeues       0           0
  acquires                    1,727,129
  releases                    0
  discards-no-route           41,171,496
  discards-ttl-expired        0
  discards-limit-overflow     0
  discards-ring-overflow      0
  discards-lvq-replace        0
  discards-subscriber-reject  0
  discards-purged             0
  reroutes                    0
  abandoned                   0
  abandoned-via-alt           0
"""


def test_qpid_stat_g():
    qpid_list = qpid_stat.QpidStatG(context_wrap(QPID_STAT_G))
    assert qpid_list.data[0].get('uptime') == '97d 0h 16m 24s'
    assert qpid_list.data[0].get('cluster') == '<standalone>'
    assert qpid_list.data[0].get('connections') == '23'
    assert qpid_list.data[0].get('sessions') == '37'
    assert qpid_list.data[0].get('exchanges') == '14'
    assert qpid_list.data[0].get('queues') == '33'
    assert qpid_list.data[1].get('Statistic') == 'queue-depth'
    assert qpid_list.data[1].get('Messages') == '0'
    assert qpid_list.data[1].get('Bytes') == '0'
    assert qpid_list.data[11].get('Statistic') == 'acquires'
    assert qpid_list.data[11].get('Messages') == '1,727,129'
    assert qpid_list.data[11].get('Bytes') == ''

    # test iteration
    assert [d.get('Messages') for d in qpid_list] == [
            None, '0', '1,727,129', '1,727,129', '28,733',
            '28,733', '0', '0', '0', '0', '0', '1,727,129',
            '0', '41,171,496', '0', '0', '0', '0', '0',
            '0', '0', '0', '0'
    ]
    assert qpid_list.by_queue['<standalone>'] == {
            'uptime': '97d 0h 16m 24s',
            'sessions': '37',
            'queues': '33',
            'connections': '23',
            'cluster': '<standalone>',
            'exchanges': '14'
    }



"""
GrubConf - The valid GRUB configuration
=======================================
Combiner for Red Hat Grub v1 and Grub v2 information.

This combiner uses the parsers:
:class:`insights.parsers.grub_conf.Grub1Config`,
:class:`insights.parsers.grub_conf.Grub1EFIConfig`,
:class:`insights.parsers.grub_conf.Grub2Config`,
:class:`insights.parsers.grub_conf.Grub2EFIConfig`, and
:class:`insights.parsers.grub_conf.BootLoaderEntries`.

It determines which parser was used by checking one of the follwing
parsers/combiners:
:class:`insights.parsers.installed_rpms.InstalledRpms`,
:class:`insights.parsers.cmdline.CmdLine`,
:class:`insights.parsers.ls_sys_firmware.LsSysFirmware`, and
:class:`insights.combiners.redhat_release.RedHatRelease`.


"""

from insights.core.plugins import combiner
from insights.combiners.redhat_release import RedHatRelease
from insights.parsers.grub_conf import BootEntry, get_kernel_initrds
from insights.parsers.grub_conf import Grub1Config, Grub1EFIConfig
from insights.parsers.grub_conf import Grub2Config, Grub2EFIConfig
from insights.parsers.grub_conf import BootLoaderEntries as BLE
from insights.parsers.ls_sys_firmware import LsSysFirmware
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.cmdline import CmdLine
from insights import SkipComponent


@combiner(BLE, optional=[LsSysFirmware])
class BootLoaderEntries(object):
    """
    Combine all :class:`insights.parsers.grub_conf.BootLoaderEntries`
    parsers into one Combiner

    Raises:
        SkipComponent: when no any BootLoaderEntries Parsers.

    Attributes:
        version (int): The version of the GRUB configuration, 1 or 2
        is_efi (bool): If the host is boot with EFI
        entries (list): List of all boot entries in GRUB configuration.
        boot_entries (list): List of all boot entries which only contains the
            name and cmdline.
        kernel_initrds (dict): Dict of the `kernel` and `initrd` files
            referenced in GRUB configuration files
        is_kdump_iommu_enabled (bool): If any kernel entry contains "intel_iommu=on"
    """
    def __init__(self, grub_bles, sys_firmware):
        self.version = self._version = 2
        self.is_efi = self._efi = '/sys/firmware/efi' in sys_firmware if sys_firmware else False
        self.entries = []
        self.boot_entries = []
        self.is_kdump_iommu_enabled = False
        for ble in grub_bles:
            self.entries.append(ble.entry)
            self.boot_entries.append(BootEntry({'name': ble.title, 'cmdline': ble.cmdline}))
            self.is_kdump_iommu_enabled = self.is_kdump_iommu_enabled or ble.is_kdump_iommu_enabled
        self.kernel_initrds = get_kernel_initrds(self.entries)

        if not self.entries:
            raise SkipComponent()


@combiner([Grub1Config, Grub2Config,
           Grub1EFIConfig, Grub2EFIConfig,
           BootLoaderEntries],
          optional=[InstalledRpms, CmdLine, LsSysFirmware, RedHatRelease])
class GrubConf(object):
    """
    Process Grub configuration v1 or v2 based on which type is passed in.

    Examples:
        >>> type(grub_conf)
        <class 'insights.combiners.grub_conf.GrubConf'>
        >>> grub_conf.version
        2
        >>> grub_conf.is_efi
        False
        >>> grub_conf.kernel_initrds
        {'grub_initrds': ['/initramfs-3.10.0-327.36.3.el7.x86_64.img'],
         'grub_kernels': ['/vmlinuz-3.10.0-327.36.3.el7.x86_64']}
        >>> grub_conf.is_kdump_iommu_enabled
        False
        >>> grub_conf.get_grub_cmdlines('')
        []

    Raises:
        Exception: when cannot find any valid grub configuration.

    Attributes:
        version (int): returns 1 or 2, version of the GRUB configuration
        is_efi (bool): returns True if the host is boot with EFI
        kernel_initrds (dict): returns a dict of the `kernel` and `initrd`
            files referenced in GRUB configuration files
        is_kdump_iommu_enabled (bool): returns True if any kernel entry
            contains "intel_iommu=on"
    """

    def __init__(self, grub1, grub2, grub1_efi, grub2_efi, grub_bles,
                 rpms, cmdline, sys_firmware, rh_rel):

        self.version = self.is_kdump_iommu_enabled = None
        self.grub = self.kernel_initrds = None
        _grubs = list(filter(None, [grub1, grub2, grub1_efi, grub2_efi, grub_bles]))
        # Check if `/sys/firmware/efi` exist?
        self.is_efi = '/sys/firmware/efi' in sys_firmware if sys_firmware else False

        if len(_grubs) == 1:
            self.grub = _grubs[0]
            self.is_efi = self.is_efi if sys_firmware else self.grub._efi
        else:
            _grub1, _grub2 = (grub1_efi, grub2_efi) if self.is_efi else (grub1, grub2)
            if rh_rel and rh_rel.rhel8:
                self.grub = grub_bles
            # Check grub version via installed-rpms
            else:
                if rpms:
                    # grub1
                    if 'grub2' not in rpms and 'grub' in rpms and _grub1 is not None:
                        self.grub = _grub1
                    # grub2
                    if 'grub' not in rpms and 'grub2' in rpms and _grub2 is not None:
                        self.grub = _grub2
                # Check grub version via the booted CmdLine
                if self.grub is None and cmdline:
                    # grub1
                    if "BOOT_IMAGE" not in cmdline or 'rd_LVM_LV' in cmdline:
                        self.grub = _grub1
                    # grub2
                    if "BOOT_IMAGE" in cmdline or 'rd.lvm.lv' in cmdline:
                        self.grub = _grub2

        if self.grub:
            self.version = self.grub._version
            self.is_kdump_iommu_enabled = self.grub.is_kdump_iommu_enabled
            self.kernel_initrds = self.grub.kernel_initrds or {}
        else:
            raise Exception('No valid grub configuration is found.')

    def get_grub_cmdlines(self, search_text=None):
        """
        Get the boot entries in which `cmdline` contains the `search_text`,
        return all the boot entries by default.

        Arguments:
            search_text(str): keyword to find in the `cmdline`, being set to
                None by default.

        Returns:
            A list of :class:`insights.parsers.grub_conf.BootEntry` objects fo
            each boot entry in which the `cmdline` contains the `search_text`.
            When `search_text` is None, returns the objects of all of the boot entries.
        """
        if search_text is None:
            return self.grub.boot_entries
        elif search_text:
            return [e for e in self.grub.boot_entries if search_text in e.get('cmdline', '')]
        return []



from insights.parsers.ovirt_engine_confd import OvirtEngineConfd
from insights.tests import context_wrap


CONFD_MATCH = """
JBOSS_HOME="/usr/share/jbossas"
ENGINE_PKI="/etc/pki/ovirt-engine"
ENGINE_PKI_CA="/etc/pki/ovirt-engine/ca.pem"
ENGINE_PKI_ENGINE_CERT="/etc/pki/ovirt-engine/certs/engine.cer"
ENGINE_TMP="${ENGINE_VAR}/tmp"
""".strip()

CONFD_NOT_MATCH = """
JBOSS_HOME="/usr/share/jbossas"
ENGINE_PKI="/etc/pki/ovirt-engine"
ENGINE_PKI_CA="/etc/pki/ovirt-engine/ca.pem"
ENGINE_PKI_ENGINE_CERT="/etc/pki/ovirt-engine/certs/engine.cer"
""".strip()


def test_ovirt_engine_confd():
    match_result = OvirtEngineConfd(context_wrap(CONFD_MATCH))
    assert 'tmp' in match_result.get('ENGINE_TMP')
    no_match_result = OvirtEngineConfd(context_wrap(CONFD_NOT_MATCH))
    assert no_match_result.get('ENGINE_TMP') is None



"""
Uptime
======

Combiner for uptime information. It uses the results of the ``Uptime``
parser and the ``Facter`` parser to get the uptime information.  ``Uptime`` is
the preferred source of data.

Examples:
    >>> ut = shared[uptime]
    >>> ut.updays
    21
    >>> ht.uptime
    03:45

"""
from __future__ import division
from collections import namedtuple
from insights.core.plugins import combiner
from insights.parsers.uptime import Uptime as upt
from insights.parsers.facter import Facter

Uptime = namedtuple("Uptime",
                    field_names=[
                        "currtime", "updays", "uphhmm",
                        "users", "loadavg", "uptime"])
"""namedtuple: Type for storing the uptime information."""


@combiner([upt, Facter])
def uptime(ut, facter):
    """Check uptime and facts to get the uptime information.

    Prefer uptime to facts.

    Returns:
        insights.combiners.uptime.Uptime: A named tuple with `currtime`,
        `updays`, `uphhmm`, `users`, `loadavg` and `uptime` components.

    Raises:
        Exception: If no data is available from both of the parsers.
    """

    ut = ut
    if ut and ut.loadavg:
        return Uptime(ut.currtime, ut.updays, ut.uphhmm,
                      ut.users, ut.loadavg, ut.uptime)
    ft = facter
    if ft and hasattr(ft, 'uptime_seconds'):
        import datetime
        secs = int(ft.uptime_seconds)
        up_dd = secs // (3600 * 24)
        up_hh = (secs % (3600 * 24)) // 3600
        up_mm = (secs % 3600) // 60
        updays = str(up_dd) if up_dd > 0 else ''
        uphhmm = '%02d:%02d' % (up_hh, up_mm)
        up_time = datetime.timedelta(seconds=secs)
        return Uptime(None, updays, uphhmm, None, None, up_time)

    raise Exception("Unable to get uptime information.")



from insights.parsers.hostname import Hostname as HnF
from insights import SkipComponent
from insights.parsers.lssap import Lssap
from insights.parsers.saphostctrl import SAPHostCtrlInstances
from insights.combiners import sap
from insights.combiners.sap import Sap
from insights.combiners.hostname import Hostname
from insights.tests import context_wrap
import pytest
import doctest

Lssap_nw_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       D16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D16/exe
  HA2|  22|       D22|         lu0416|749, patch 10, changelist 1698137|          /usr/sap/HA2/D22/exe
  HA2|  50|       D50|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D50/exe
  HA2|  51|       D51|         lu0418|749, patch 10, changelist 1698137|          /usr/sap/HA2/D51/exe
  HA2|  52|       D52|         lu0418|749, patch 10, changelist 1698137|          /usr/sap/HA2/D52/exe
==========================================
""".strip()

Lssap_hana_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       HDB16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/HDB16/exe
==========================================
""".strip()

Lssap_ascs_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       ASCS16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/ASCS16/exe
==========================================
""".strip()

Lssap_all_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       D16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D16/exe
  HA2|  22|       D22|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D22/exe
  HA2|  16|       HDB16|       lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/HDB16/exe
  HA2|  16|       ASCS16|      lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/ASCS16/exe
==========================================
""".strip()

Lssap_doc_TEST = """
 - lssap version 1.0 -
==========================================
  SID   Nr   Instance    SAPLOCALHOST                        Version                 DIR_EXECUTABLE
  HA2|  16|       D16|         lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/D16/exe
  HA2|  22|       D22|         lu0418|749, patch 10, changelist 1698137|          /usr/sap/HA2/D22/exe
  HA2|  16|       HDB16|       lu0417|749, patch 10, changelist 1698137|          /usr/sap/HA2/HDB16/exe
==========================================
""".strip()

HOSTNAME = 'lu0417.example.com'
HOSTNAME1 = 'li-ld-1810.example.com'

SAPHOSTCTRL_HOSTINSTANCES = '''
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D89
 SystemNumber , String , 88
 InstanceName , String , HDB88
 Hostname , String , lu0417
 FullQualifiedHostname , String , lu0417.example.com
 IPAddress , String , 10.0.0.88
 SapVersionInfo , String , 749, patch 211, changelist 1754007
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D90
 SystemNumber , String , 90
 InstanceName , String , HDB90
 Hostname , String , lu0418
 FullQualifiedHostname , String , lu0418.example.com
 IPAddress , String , 10.0.0.90
 SapVersionInfo , String , 749, patch 211, changelist 1754007
'''

SAPHOSTCTRL_HOSTINSTANCES_GOOD = '''
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D89
 SystemNumber , String , 88
 InstanceName , String , HDB88
 Hostname , String , li-ld-1810
 FullQualifiedHostname , String , li-ld-1810.example.com
 IPAddress , String , 10.0.0.1
 SapVersionInfo , String , 749, patch 211, changelist 1754007
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D90
 SystemNumber , String , 90
 InstanceName , String , HDB90
 Hostname , String , li-ld-1810
 FullQualifiedHostname , String , li-ld-1810.example.com
 IPAddress , String , 10.0.0.1
 SapVersionInfo , String , 749, patch 211, changelist 1754007
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D79
 SystemNumber , String , 08
 InstanceName , String , ERS08
 Hostname , String , d79ers
 FullQualifiedHostname , String , d79ers.example.com
 IPAddress , String , 10.0.0.15
 SapVersionInfo , String , 749, patch 301, changelist 1779613
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D79
 SystemNumber , String , 07
 InstanceName , String , ASCS07
 Hostname , String , d79ascs
 FullQualifiedHostname , String , d79ascs.example.com
 IPAddress , String , 10.0.0.14
 SapVersionInfo , String , 749, patch 301, changelist 1779613
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D79
 SystemNumber , String , 09
 InstanceName , String , DVEBMGS09
 Hostname , String , d79pas
 FullQualifiedHostname , String , d79pas.example.com
 IPAddress , String , 10.0.0.16
 SapVersionInfo , String , 749, patch 301, changelist 1779613
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D80
 SystemNumber , String , 10
 InstanceName , String , SCS10
 Hostname , String , d80scs
 FullQualifiedHostname , String , d80scs.example.com
 IPAddress , String , 10.0.0.17
 SapVersionInfo , String , 749, patch 301, changelist 1779613
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D62
 SystemNumber , String , 62
 InstanceName , String , HDB62
 Hostname , String , d62dbsrv
 FullQualifiedHostname , String , li-ld-1810.example.com
 IPAddress , String , 10.0.1.12
 SapVersionInfo , String , 749, patch 211, changelist 1754007
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D52
 SystemNumber , String , 52
 InstanceName , String , ASCS52
 Hostname , String , d52ascs
 FullQualifiedHostname , String , d52ascs.example.com
 IPAddress , String , 10.0.0.20
 SapVersionInfo , String , 749, patch 401, changelist 1806777
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D52
 SystemNumber , String , 54
 InstanceName , String , D54
 Hostname , String , d52pas
 FullQualifiedHostname , String , d52pas.example.com
 IPAddress , String , 10.0.0.22
 SapVersionInfo , String , 749, patch 401, changelist 1806777
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , SMA
 SystemNumber , String , 91
 InstanceName , String , SMDA91
 Hostname , String , li-ld-1810
 FullQualifiedHostname , String , li-ld-1810.example.com
 IPAddress , String , 10.0.0.1
 SapVersionInfo , String , 749, patch 200, changelist 1746260
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , B15
 SystemNumber , String , 00
 InstanceName , String , HDB00
 Hostname , String , sapb15hdba1
 FullQualifiedHostname , String , li-ld-1810.example.com
 SapVersionInfo , String , 749, patch 418, changelist 1816226
*********************************************************
'''


def test_lssap_netweaver():
    lssap = Lssap(context_wrap(Lssap_nw_TEST))
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None, None)
    sap = Sap(hn, None, lssap)
    assert sap['D50'].number == '50'
    assert 'D16' in sap.local_instances
    assert 'D51' in sap.all_instances
    assert 'D51' not in sap.local_instances
    assert sap.is_netweaver is True
    assert sap.is_hana is False
    assert sap.is_ascs is False


def test_saphostcrtl_hana():
    lssap = Lssap(context_wrap(Lssap_nw_TEST))
    inst = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES))
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None, None)
    sap = Sap(hn, inst, lssap)
    assert 'D50' not in sap
    assert sap.local_instances == ['HDB88']
    assert 'HDB90' in sap.all_instances
    assert sap['HDB88'].number == '88'
    assert sap['HDB90'].hostname == 'lu0418'
    assert sap['HDB90'].version == '749, patch 211, changelist 1754007'
    assert sap.number('HDB90') == '90'
    assert sap.sid('HDB88') == 'D89'
    assert sap.is_netweaver is False
    assert sap.is_hana is True
    assert sap.is_ascs is False


def test_saphostcrtl_hana_2():
    lssap = Lssap(context_wrap(Lssap_all_TEST))
    inst = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_GOOD))
    hn = Hostname(HnF(context_wrap(HOSTNAME1)), None, None, None, None)
    sap = Sap(hn, inst, lssap)
    assert 'D50' not in sap
    assert 'HDB00' in sap
    assert sorted(sap.local_instances) == sorted(['HDB88', 'HDB90', 'SMDA91'])
    assert sorted(sap.all_instances) == sorted([
        'ASCS07', 'ASCS52', 'D54', 'DVEBMGS09', 'ERS08', 'HDB00', 'HDB62',
        'HDB88', 'HDB90', 'SCS10', 'SMDA91'])
    assert sorted(sap.business_instances) == sorted([
        'ASCS07', 'ASCS52', 'D54', 'DVEBMGS09', 'ERS08', 'HDB00', 'HDB62',
        'HDB88', 'HDB90', 'SCS10'])
    assert sorted(sap.function_instances) == sorted(['SMDA91'])
    assert sap['HDB88'].number == '88'
    assert sap['HDB90'].hostname == 'li-ld-1810'
    assert sap['DVEBMGS09'].version == '749, patch 301, changelist 1779613'
    assert sap.version('HDB90') == '749, patch 211, changelist 1754007'
    assert sap.hostname('HDB62') == 'd62dbsrv'
    assert sap.type('SCS10') == 'SCS'
    assert sap.is_netweaver is True
    assert sap.is_hana is True
    assert sap.is_ascs is True


def test_lssap_hana():
    lssap = Lssap(context_wrap(Lssap_hana_TEST))
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None, None)
    sap = Sap(hn, None, lssap)
    assert 'D50' not in sap
    assert sap.is_netweaver is False
    assert sap.is_hana is True
    assert sap.is_ascs is False


def test_lssap_ascs():
    lssap = Lssap(context_wrap(Lssap_ascs_TEST))
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None, None)
    sap = Sap(hn, None, lssap)
    assert sap['ASCS16'].sid == 'HA2'
    # ASCS is also a kind of NetWeaver
    assert sap.is_netweaver is True
    assert sap.is_hana is False
    assert sap.is_ascs is True


def test_all():
    lssap = Lssap(context_wrap(Lssap_all_TEST))
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None, None)
    sap = Sap(hn, None, lssap)
    assert sap['D16'].version == '749, patch 10, changelist 1698137'
    assert sap['ASCS16'].hostname == 'lu0417'
    assert sap.is_netweaver is True
    assert sap.is_hana is True
    assert sap.is_ascs is True


def test_doc_examples():
    env = {
            'saps': Sap(
                Hostname(HnF(context_wrap(HOSTNAME)), None, None, None, None),
                None,
                Lssap(context_wrap(Lssap_doc_TEST))
            )
          }
    failed, total = doctest.testmod(sap, globs=env)
    assert failed == 0


def test_ab():
    hn = Hostname(HnF(context_wrap(HOSTNAME)), None, None, None, None)
    with pytest.raises(SkipComponent) as se:
        Sap(hn, None, None)
    assert 'No SAP instance.' in str(se)



from insights.parsers import ls_edac_mc
from insights.parsers.ls_edac_mc import LsEdacMC
from insights.tests import context_wrap
import doctest

LS_EDAC_MC = """
/sys/devices/system/edac/mc:
total 37592
drwxr-xr-x. 3 0 0 0 Jan 10 10:33 .
drwxr-xr-x. 3 0 0 0 Jan 10 10:33 ..
drwxr-xr-x. 2 0 0 0 Jan 10 10:33 power
drwxr-xr-x. 2 0 0 0 Jan 10 10:33 mc0
drwxr-xr-x. 2 0 0 0 Jan 10 10:33 mc1
drwxr-xr-x. 2 0 0 0 Jan 10 10:33 mc2
"""


def test_ls_edac_mc():
    ls_edac_mc = LsEdacMC(context_wrap(LS_EDAC_MC))
    assert '/sys/devices/system/edac/mc' in ls_edac_mc
    assert ls_edac_mc.dirs_of('/sys/devices/system/edac/mc') == ['.', '..', 'power', 'mc0', 'mc1', 'mc2']


def test_ls_etc_documentation():
    failed_count, tests = doctest.testmod(
        ls_edac_mc,
        globs={'ls_edac_mc': ls_edac_mc.LsEdacMC(context_wrap(LS_EDAC_MC))}
    )
    assert failed_count == 0



from insights.tests import context_wrap
from insights.parsers.grub_conf import Grub2EFIConfig

GRUB2_EFI_CFG = """
### BEGIN /etc/grub.d/10_linux ###
menuentry 'Red Hat Enterprise Linux Server (3.10.0-514.16.1.el7.x86_64) 7.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-514.el7.x86_64-advanced-9727cab4-12c2-41a8-9527-9644df34e586' {
    load_video
    set gfxpayload=keep
    insmod gzio
    insmod part_gpt
    insmod xfs
    set root='hd0,gpt2'
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,gpt2 --hint-efi=hd0,gpt2 --hint-baremetal=ahci0,gpt2  d80fa96c-ffa1-4894-9282-aeda37f0befe
    else
      search --no-floppy --fs-uuid --set=root d80fa96c-ffa1-4894-9282-aeda37f0befe
    fi
    linuxefi /vmlinuz-3.10.0-514.16.1.el7.x86_64 root=/dev/mapper/rhel-root ro rd.luks.uuid=luks-a40b320e-0711-4cd6-8f9e-ce32810e2a79 rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8
    initrdefi /initramfs-3.10.0-514.16.1.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server (3.10.0-514.10.2.el7.x86_64) 7.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-514.el7.x86_64-advanced-9727cab4-12c2-41a8-9527-9644df34e586' {
    load_video
    set gfxpayload=keep
    insmod gzio
    insmod part_gpt
    insmod xfs
    set root='hd0,gpt2'
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,gpt2 --hint-efi=hd0,gpt2 --hint-baremetal=ahci0,gpt2  d80fa96c-ffa1-4894-9282-aeda37f0befe
    else
      search --no-floppy --fs-uuid --set=root d80fa96c-ffa1-4894-9282-aeda37f0befe
    fi
    linuxefi /vmlinuz-3.10.0-514.10.2.el7.x86_64 root=/dev/mapper/rhel-root ro rd.luks.uuid=luks-a40b320e-0711-4cd6-8f9e-ce32810e2a79 rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8
    initrdefi /initramfs-3.10.0-514.10.2.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server (3.10.0-514.el7.x86_64) 7.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-514.el7.x86_64-advanced-9727cab4-12c2-41a8-9527-9644df34e586' {
    load_video
    set gfxpayload=keep
    insmod gzio
    insmod part_gpt
    insmod xfs
    set root='hd0,gpt2'
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,gpt2 --hint-efi=hd0,gpt2 --hint-baremetal=ahci0,gpt2  d80fa96c-ffa1-4894-9282-aeda37f0befe
    else
      search --no-floppy --fs-uuid --set=root d80fa96c-ffa1-4894-9282-aeda37f0befe
    fi
    linuxefi /vmlinuz-3.10.0-514.el7.x86_64 root=/dev/mapper/rhel-root ro rd.luks.uuid=luks-a40b320e-0711-4cd6-8f9e-ce32810e2a79 rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet LANG=en_US.UTF-8
    initrdefi /initramfs-3.10.0-514.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server (0-rescue-f1340b5dd6ee4c26b587621566111421) 7.3 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-0-rescue-f1340b5dd6ee4c26b587621566111421-advanced-9727cab4-12c2-41a8-9527-9644df34e586' {
    load_video
    insmod gzio
    insmod part_gpt
    insmod xfs
    set root='hd0,gpt2'
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,gpt2 --hint-efi=hd0,gpt2 --hint-baremetal=ahci0,gpt2  d80fa96c-ffa1-4894-9282-aeda37f0befe
    else
      search --no-floppy --fs-uuid --set=root d80fa96c-ffa1-4894-9282-aeda37f0befe
    fi
    linuxefi /vmlinuz-0-rescue-f1340b5dd6ee4c26b587621566111421 root=/dev/mapper/rhel-root ro rd.luks.uuid=luks-a40b320e-0711-4cd6-8f9e-ce32810e2a79 rd.lvm.lv=rhel/root rd.lvm.lv=rhel/swap rhgb quiet
    initrdefi /initramfs-0-rescue-f1340b5dd6ee4c26b587621566111421.img
}

### END /etc/grub.d/10_linux ###
""".strip()  # noqa


class TestGrub2EFI():
    def test_get_grub_kernel_initrd(self):
        expected = {'grub_kernels': ['vmlinuz-3.10.0-514.16.1.el7.x86_64',
                                     'vmlinuz-3.10.0-514.10.2.el7.x86_64',
                                     'vmlinuz-3.10.0-514.el7.x86_64',
                                     'vmlinuz-0-rescue-f1340b5dd6ee4c26b587621566111421'],
                    'grub_initrds': ['initramfs-3.10.0-514.16.1.el7.x86_64.img',
                                     'initramfs-3.10.0-514.10.2.el7.x86_64.img',
                                     'initramfs-3.10.0-514.el7.x86_64.img',
                                     'initramfs-0-rescue-f1340b5dd6ee4c26b587621566111421.img']}

        assert expected == Grub2EFIConfig((context_wrap(GRUB2_EFI_CFG))).kernel_initrds



"""
dirsrv_sysconfig - file ``/etc/sysconfig/dirsrv``
=================================================

This module provides the ``DirsrvSysconfig`` class parser, for reading the
options in the ``/etc/sysconfig/dirsrv`` file.

Sample input::

    # how many seconds to wait for the startpid file to show
    # up before we assume there is a problem and fail to start
    # if using systemd, omit the "; export VARNAME" at the end
    #STARTPID_TIME=10 ; export STARTPID_TIME
    # how many seconds to wait for the pid file to show
    # up before we assume there is a problem and fail to start
    # if using systemd, omit the "; export VARNAME" at the end
    #PID_TIME=600 ; export PID_TIME
    KRB5CCNAME=/tmp/krb5cc_995
    KRB5_KTNAME=/etc/dirsrv/ds.keytab

Examples:

    >>> dirsrv_conf = shared[DirsrvSysconfig]
    >>> dirsrv.KRB5_KTNAME
    '/etc/dirsrv/ds.keytab'
    >>> 'PID_TIME' in dirsrv.data
    False

"""
from insights.util import deprecated
from .. import parser, SysconfigOptions
from insights.specs import Specs


@parser(Specs.dirsrv)
class DirsrvSysconfig(SysconfigOptions):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.sysconfig.DirsrvSysconfig` instead.

    Parse the `dirsrv` service's start-up configuration.
    """
    def __init__(self, *args, **kwargs):
        deprecated(DirsrvSysconfig, "Import DirsrvSysconfig from insights.parsers.sysconfig instead")
        super(DirsrvSysconfig, self).__init__(*args, **kwargs)

    set_properties = True



"""
Manila configuration - file ``/etc/manila/manila.conf``
=======================================================

The Manila configuration file is a standard '.ini' file and this parser uses
the ``IniConfigFile`` class to read it.

Sample configuration::

    [DEFAULT]
    osapi_max_limit = 1000
    osapi_share_base_URL = <None>

    use_forwarded_for = false
    api_paste_config = api-paste.ini
    state_path = /var/lib/manila

    scheduler_topic = manila-scheduler

    share_topic = manila-share

    share_driver = manila.share.drivers.generic.GenericShareDriver

    enable_v1_api = false
    enable_v2_api = false

    [cors]
    allowed_origin = <None>
    allow_credentials = true

    expose_headers = Content-Type,Cache-Control,Content-Language,Expires,Last-Modified,Pragma
    allow_methods = GET,POST,PUT,DELETE,OPTIONS
    allow_headers = Content-Type,Cache-Control,Content-Language,Expires,Last-Modified,Pragma

Examples:

    >>> conf = shared[ManilaConf]
    >>> conf.sections()
    ['DEFAULT', 'cors']
    >>> 'cors' in conf
    True
    >>> conf.has_option('DEFAULT', 'share_topic')
    True
    >>> conf.get("DEFAULT", "share_topic")
    "manila-share"
    >>> conf.get("DEFAULT", "enable_v2_api")
    "false"
    >>> conf.getboolean("DEFAULT", "enable_v2_api")
    False
    >>> conf.getint("DEFAULT", "osapi_max_limit")
    1000

"""

from .. import parser, IniConfigFile
from insights.specs import Specs


@parser(Specs.manila_conf)
class ManilaConf(IniConfigFile):
    """
    Manila configuration parser class, based on the ``IniConfigFile`` class.
    """
    pass



from insights.parsers.systemd import config
from insights.parsers import SkipException
from insights.tests import context_wrap
from insights.core.plugins import ContentException
import doctest
import pytest


SYSTEMD_DOCKER = """
[Unit]
Description=Docker Application Container Engine
Documentation=http://docs.docker.com
After=network.target
Wants=docker-storage-setup.service
Requires=rhel-push-plugin.socket

[Service]
Type=notify
NotifyAccess=all
EnvironmentFile=-/etc/sysconfig/docker
EnvironmentFile=-/etc/sysconfig/docker-storage
EnvironmentFile=-/etc/sysconfig/docker-network
Environment=GOTRACEBACK=crash
ExecStart=/bin/sh -c '/usr/bin/docker-current daemon \\
          --authorization-plugin=rhel-push-plugin \\
          --exec-opt native.cgroupdriver=systemd \\
          $OPTIONS \\
          $DOCKER_STORAGE_OPTIONS \\
          $DOCKER_NETWORK_OPTIONS \\
          $ADD_REGISTRY \\
          $BLOCK_REGISTRY \\
          $INSECURE_REGISTRY \\
          2>&1 | /usr/bin/forward-journald -tag docker'
LimitNOFILE=1048576
LimitNPROC=1048576
LimitCORE=infinity
TimeoutStartSec=0
MountFlags=slave
Restart=on-abnormal
StandardOutput=null
StandardError=null

[Install]
WantedBy=multi-user.target
""".strip()


SYSTEMD_DOCKER_EMPTY = """
Unit docker.service is not loaded: No such file or directory
""".strip()


SYSTEMD_OPENSHIFT_NODE = """
[Unit]
Description=Atomic OpenShift Node
After=docker.service
After=openvswitch.service
Wants=docker.service
Documentation=https://github.com/openshift/origin

[Service]
Type=notify
EnvironmentFile=/etc/sysconfig/atomic-openshift-node
Environment=GOTRACEBACK=crash
ExecStart=/usr/bin/openshift start node --config=${CONFIG_FILE} $OPTIONS
LimitNOFILE=65536
LimitCORE=infinity
WorkingDirectory=/var/lib/origin/
SyslogIdentifier=atomic-openshift-node
Restart=always
RestartSec=5s
OOMScoreAdjust=-999
ExecStartPost=/usr/bin/sleep 10
ExecStartPost=/usr/sbin/sysctl --system

[Install]
WantedBy=multi-user.target

""".strip()


SYSTEMD_LOGIND_CONF = """
#  This file is part of systemd.
#
#  systemd is free software; you can redistribute it and/or modify it
#  under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 2.1 of the License, or
#  (at your option) any later version.
#
# Entries in this file show the compile time defaults.
# You can change settings by editing this file.
# Defaults can be restored by simply deleting this file.
#
# See logind.conf(5) for details.

[Login]
#NAutoVTs=6
ReserveVT=6
KillUserProcesses=Yes
#KillOnlyUsers=
#KillExcludeUsers=root
#InhibitDelayMaxSec=5
#HandlePowerKey=poweroff
#HandleSuspendKey=suspend
#HandleHibernateKey=hibernate
#HandleLidSwitch=suspend
#HandleLidSwitchDocked=ignore
#PowerKeyIgnoreInhibited=no
#SuspendKeyIgnoreInhibited=no
#HibernateKeyIgnoreInhibited=no
#LidSwitchIgnoreInhibited=yes
#IdleAction=ignore
#IdleActionSec=30min
RuntimeDirectorySize=10%
RemoveIPC=no
#UserTasksMax=
""".strip()

SYSTEMD_RPCBIND_SOCKET = """
[Unit]
Description=RPCbind Server Activation Socket
DefaultDependencies=no
Wants=rpcbind.target
Before=rpcbind.target

[Socket]
ListenStream=/run/rpcbind.sock

# RPC netconfig can't handle ipv6/ipv4 dual sockets
BindIPv6Only=ipv6-only
ListenStream=0.0.0.0:111
ListenDatagram=0.0.0.0:111
ListenStream=[::]:111
ListenDatagram=[::]:111

[Install]
WantedBy=sockets.target
""".strip()

SYSTEMD_SYSTEM_CONF = """
#  This file is part of systemd.
#
#  systemd is free software; you can redistribute it and/or modify it
#  under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 2.1 of the License, or
#  (at your option) any later version.
#
# Entries in this file show the compile time defaults.
# You can change settings by editing this file.
# Defaults can be restored by simply deleting this file.
#
# See systemd-system.conf(5) for details.

[Manager]
#LogLevel=info
#LogTarget=journal-or-kmsg
#LogColor=yes
#LogLocation=no
#DumpCore=yes
#CrashShell=no
#ShowStatus=yes
#CrashChVT=1
#CPUAffinity=1 2
#JoinControllers=cpu,cpuacct net_cls,net_prio
RuntimeWatchdogSec=0
ShutdownWatchdogSec=10min
#CapabilityBoundingSet=
#SystemCallArchitectures=
#TimerSlackNSec=
#DefaultTimerAccuracySec=1min
#DefaultStandardOutput=journal
#DefaultStandardError=inherit
#DefaultTimeoutStartSec=90s
#DefaultTimeoutStopSec=90s
#DefaultRestartSec=100ms
#DefaultStartLimitInterval=10s
#DefaultStartLimitBurst=5
#DefaultEnvironment=
#DefaultCPUAccounting=no
#DefaultBlockIOAccounting=no
#DefaultMemoryAccounting=no
#DefaultLimitCPU=
#DefaultLimitFSIZE=
#DefaultLimitDATA=
#DefaultLimitSTACK=
#DefaultLimitCORE=
#DefaultLimitRSS=
#DefaultLimitNOFILE=
#DefaultLimitAS=
#DefaultLimitNPROC=
#DefaultLimitMEMLOCK=
#DefaultLimitLOCKS=
#DefaultLimitSIGPENDING=
#DefaultLimitMSGQUEUE=
#DefaultLimitNICE=
#DefaultLimitRTPRIO=
#DefaultLimitRTTIME=
""".strip()

SYSTEMD_SYSTEM_ORIGIN_ACCOUNTING = """
[Manager]
DefaultCPUAccounting=yes
DefaultMemoryAccounting=yes
# systemd v230 or newer
DefaultIOAccounting=yes
# Deprecated, remove in future
DefaultBlockIOAccounting=no
""".strip()


def test_systemd_docker():
    docker_service = config.SystemdDocker(context_wrap(SYSTEMD_DOCKER))
    assert docker_service.data["Unit"]["After"] == "network.target"
    assert docker_service.data["Service"]["NotifyAccess"] == "all"
    assert docker_service.data["Service"]["Environment"] == "GOTRACEBACK=crash"
    assert docker_service.data["Install"]["WantedBy"] == "multi-user.target"
    assert list(docker_service.data["Install"].keys()) == ["WantedBy"]
    assert docker_service.data["Service"]["ExecStart"] == "/bin/sh -c '/usr/bin/docker-current daemon --authorization-plugin=rhel-push-plugin --exec-opt native.cgroupdriver=systemd $OPTIONS $DOCKER_STORAGE_OPTIONS $DOCKER_NETWORK_OPTIONS $ADD_REGISTRY $BLOCK_REGISTRY $INSECURE_REGISTRY 2>&1 | /usr/bin/forward-journald -tag docker'"


def test_systemd_docker_empty():
    with pytest.raises(ContentException):
        config.SystemdDocker(context_wrap(SYSTEMD_DOCKER_EMPTY))


def test_systemd_openshift_node():
    openshift_node_service = config.SystemdOpenshiftNode(context_wrap(SYSTEMD_OPENSHIFT_NODE))
    assert openshift_node_service.data["Unit"]["Wants"] == "docker.service"
    assert openshift_node_service.data["Unit"]["After"] == ['docker.service', 'openvswitch.service']


def test_systemd_system_conf():
    common_conf_info = config.SystemdSystemConf(context_wrap(SYSTEMD_SYSTEM_CONF))
    assert "Manager" in common_conf_info
    print(common_conf_info.doc)
    assert common_conf_info["Manager"]["RuntimeWatchdogSec"] == "0"
    assert common_conf_info["Manager"]["ShutdownWatchdogSec"] == "10min"


def test_systemd_system_origin_accounting():
    common_system_origin_accounting = config.SystemdOriginAccounting(context_wrap(SYSTEMD_SYSTEM_ORIGIN_ACCOUNTING))
    assert "Manager" in common_system_origin_accounting
    assert common_system_origin_accounting["Manager"]["DefaultCPUAccounting"] == 'True'
    assert common_system_origin_accounting["Manager"]["DefaultBlockIOAccounting"] == 'False'


def test_systemd_logind_conf():
    logind_conf = config.SystemdLogindConf(context_wrap(SYSTEMD_LOGIND_CONF))
    assert "Login" in logind_conf
    assert logind_conf["Login"]["RemoveIPC"] == "False"
    assert logind_conf["Login"]["RuntimeDirectorySize"] == "10%"


def test_systemd_rpcbind_socket_conf():
    rpcbind_socket = config.SystemdRpcbindSocketConf(context_wrap(SYSTEMD_RPCBIND_SOCKET))
    assert "Socket" in rpcbind_socket
    assert rpcbind_socket["Socket"]["ListenStream"] == ['/run/rpcbind.sock', '0.0.0.0:111', '[::]:111']
    assert rpcbind_socket["Socket"]["ListenDatagram"] == ['0.0.0.0:111', '[::]:111']


def test_systemd_empty():
    with pytest.raises(SkipException):
        assert config.SystemdLogindConf(context_wrap('')) is None


def test_doc_examples():
    env = {
            'docker_service': config.SystemdDocker(context_wrap(SYSTEMD_DOCKER)),
            'system_conf': config.SystemdSystemConf(context_wrap(SYSTEMD_SYSTEM_CONF)),
            'system_origin_accounting': config.SystemdOriginAccounting(context_wrap(SYSTEMD_SYSTEM_ORIGIN_ACCOUNTING)),
            'openshift_node_service': config.SystemdOpenshiftNode(context_wrap(SYSTEMD_OPENSHIFT_NODE)),
            'logind_conf': config.SystemdLogindConf(context_wrap(SYSTEMD_LOGIND_CONF)),
            'rpcbind_socket': config.SystemdRpcbindSocketConf(context_wrap(SYSTEMD_RPCBIND_SOCKET))
          }
    failed, total = doctest.testmod(config, globs=env)
    assert failed == 0



#!/usr/bin/env python
"""
Standaone Rule
==============

This is a customer spec, parser and rule and can be run
against the local host using the following command::

    $ insights-run -p examples.rules.stand_alone

or from the examples/rules directory::

    $ ./stand_alone.py
"""
from __future__ import print_function
from collections import namedtuple

from insights import get_active_lines, parser, Parser
from insights import make_fail, make_pass, rule, run
from insights.core.spec_factory import SpecSet, simple_file
from insights.parsers.redhat_release import RedhatRelease

# Error key used in make_fail
ERROR_KEY = "TOO_MANY_HOSTS"

# jinga2 template displayed for rule responses

CONTENT = {
    make_fail: """Too many hosts in /etc/hosts: {{num}}""",
    make_pass: """Just right"""
}


class Specs(SpecSet):
    """ Datasources for collection from local host """
    hosts = simple_file("/etc/hosts")


@parser(Specs.hosts)
class HostParser(Parser):
    """
    Parses the results of the ``hosts`` Specs

    Attributes:
        hosts (list): List of the namedtuple Host
            which are the contents of the hosts file
            including ``.ip``, ``.host``, and ``.aliases``.
    """
    Host = namedtuple("Host", ["ip", "host", "aliases"])

    def parse_content(self, content):
        """
        Method to parse the contents of file ``/etc/hosts``

        This method must be implemented by each parser.

        Arguments:
            content (list): List of strings that are the contents
                of the /etc/hosts file.
        """
        self.hosts = []
        for line in get_active_lines(content):
            # remove inline comments
            line = line.partition("#")[0].strip()

            # break the line into parts
            parts = line.split()
            ip, host = parts[:2]
            aliases = parts[2:]

            self.hosts.append(HostParser.Host(ip, host, aliases))

    def __repr__(self):
        """ str: Returns string representation of the class """
        me = self.__class__.__name__
        msg = "%s([" + ", ".join([str(d) for d in self.hosts]) + "])"
        return msg % me


@rule(HostParser, RedhatRelease, content=CONTENT)
def report(hp, rhr):
    """
    Rule reports a response if there is more than 1 host
    entry defined in the /etc/hosts file.

    Arguments:
        hp (HostParser): Parser object for the custom parser in this
            module.
        rhr (RedhatRelease): Parser object for the /etc/redhat-release
            file.
    """
    if len(hp.hosts) > 1:
        return make_fail("TOO_MANY_HOSTS", num=len(hp.hosts))
    return make_pass("TOO_MANY_HOSTS", num=len(hp.hosts))


if __name__ == "__main__":
    run(report, print_summary=True)



from insights.parsers.foreman_proxy_conf import ForemanProxyConf
from insights.tests import context_wrap

conf_content = """
---
### File managed with puppet ###
## Module:           'foreman_proxy'

:settings_directory: /etc/foreman-proxy/settings.d

# SSL Setup

# if enabled, all communication would be verfied via SSL
# NOTE that both certificates need to be signed by the same CA in order for this to work
# see http://theforeman.org/projects/smart-proxy/wiki/SSL for more information
:ssl_ca_file: /etc/foreman-proxy/ssl_ca.pem
:ssl_certificate: /etc/foreman-proxy/ssl_cert.pem
:ssl_private_key: /etc/foreman-proxy/ssl_key.pem

# the hosts which the proxy accepts connections from
# commenting the following lines would mean every verified SSL connection allowed
:trusted_hosts:
 - xxx-eopv.xxx.com
 - xxx-eopv.xxx.com

# Endpoint for reverse communication
:foreman_url: https://xxx-eopv.xxx.com
""".strip()


def test_settings_yml():
    result = ForemanProxyConf(context_wrap(conf_content))
    assert result.data[':settings_directory'] == '/etc/foreman-proxy/settings.d'
    assert result.data[':ssl_ca_file'] == '/etc/foreman-proxy/ssl_ca.pem'
    assert result.data[':ssl_private_key'] == '/etc/foreman-proxy/ssl_key.pem'
    assert result.data[':foreman_url'] == 'https://xxx-eopv.xxx.com'
    assert result.data[':trusted_hosts'] == ['xxx-eopv.xxx.com', 'xxx-eopv.xxx.com']
    assert "xxx-eopv.xxx.com" in result.data[':trusted_hosts']



"""
PackageProvidesJava - command ``/bin/echo {java_command_package}``
==================================================================

This command reads the output of the pre-command:

``for jp in `/bin/ps auxwww | grep java | grep -v grep| awk '{print $11}' | sort -u`; do echo $jp `readlink -e $jp | xargs rpm -qf`; done``

This command looks for all versions of 'java' running and tries to find the
RPM packages which provide them.  The running command and its package name
are stored as properties ``command`` and ``package`` of the object.

The reason why using above pre_command is that we need to record the links
between running_java_command and package which provides the java command. In
``ps aux`` output, we can only get what java command starts a java
application, instead of java package. Through this way, when there is jdk
bug, we can detect whether a running java application will be affected.

Typical contents of the pre_command::

    /usr/lib/jvm/jre/bin/java java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64

Parsed result::

    self.command = "/usr/lib/jvm/jre/bin/java"
    self.package = "java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64"

Examples:

    >>> command_package = shared[PackageProvidesJava]
    >>> command_package.command
    '/usr/lib/jvm/jre/bin/java'
    >>> command_package.package
    'java-1.8.0-openjdk-headless-1.8.0.141-3.b16.el6_9.x86_64'

Raises:
    insights.parsers.ParseException: if there is no java application running

Raises:
    insights.parsers.SkipException: if running java command is not provided by package installed through yum or rpm
"""

from insights import parser, CommandParser
from ..parsers import ParseException, SkipException
from insights.specs import Specs


@parser(Specs.package_provides_java)
class PackageProvidesJava(CommandParser):
    """
    Parse the output of pre_command::

        ``for jp in `/bin/ps auxwww | grep java | grep -v grep| awk '{print $11}' | sort -u`; do echo "$jp `readlink -e $jp | xargs rpm -qf`"; done``.

    Attributes:
        command (str): The java command that starts application.
        package (str): Java package that provides above java command.
    """

    def parse_content(self, content):
        if len(content) == 0:
            raise ParseException("Error: ", 'there is not java application running')
        l = content[0].split()
        if len(l) != 2:
            raise SkipException("Error: ",
                                'current running java command is not provided by package installed through yum or rpm')
        self.command = l[0]
        self.package = l[1]



"""
Cobbler modules configuration - file ``/etc/cobbler/modules.conf``
==================================================================

The Cobbler modules configuration lists a set of services, and typically
sets the module that provides that service.

Sample input::

    [authentication]
    module = authn_spacewalk

    [authorization]
    module = authz_allowall

    [dns]
    module = manage_bind

    [dhcp]
    module = manage_isc

Examples:

    >>> conf = CobblerModulesConf(context_wrap(conf_content))
    >>> conf.get('authentication', 'module')
    'authn_spacewalk'
    >>> conf.get('dhcp', 'module')
    'manage_isc'

"""

from .. import parser, IniConfigFile
from insights.specs import Specs


@parser(Specs.cobbler_modules_conf)
class CobblerModulesConf(IniConfigFile):
    """
    This uses the standard ``IniConfigFile`` parser class.
    """
    pass



#!/usr/bin/env python
from __future__ import print_function

import argparse
import os
import re


def mangle_command(command, name_max=255, has_variables=False):
    """
    Mangle a command line string into something suitable for use as the basename of a filename.
    At minimum this function must remove slashes, but it also does other things to clean up
    the basename: removing directory names from the command name, replacing many non-typical
    characters with undersores, in addition to replacing slashes with dots.

    By default, curly braces, '{' and '}', are replaced with underscore, set 'has_variables'
    to leave curly braces alone.

    This function was copied from the function that insights-client uses to create the name it uses
    to capture the output of the command.

    Here, server side, it is used to figure out what file in the archive contains the output of
    a command.  Server side, the command may contain references to variables (names within
    matching curly braces) that will be expanded before the name is actually used as a file name.

    To completly mimic the insights-client behavior, curly braces need to be replaced with
    underscores.  If the command has variable references, the curly braces must be left alone.
    Set has_variables, to leave curly braces alone.

    This implementation of 'has_variables' assumes that variable names only contain characters
    that are not replaced by mangle_command.
    """
    if has_variables:
        pattern = r"[^\w\-\.\/{}]+"
    else:
        pattern = r"[^\w\-\.\/]+"

    mangledname = re.sub(r"^/(usr/|)(bin|sbin)/", "", command)
    mangledname = re.sub(pattern, "_", mangledname)
    mangledname = re.sub(r"/", ".", mangledname).strip(" ._-")
    mangledname = mangledname[:name_max]
    return mangledname


def parse_args():
    p = argparse.ArgumentParser()
    p.add_argument("command", help="Command string to mangle.")
    return p.parse_args()


def main():
    args = parse_args()
    print(os.path.join("insights_commands", mangle_command(args.command)))


if __name__ == "__main__":
    main()



import json
from insights import specs, datasource
from insights.core.spec_factory import DatasourceProvider
from . import GVK, foreach_resource, resource


def cr_gvk(crd):
    group = crd["spec"]["group"]
    version = crd["spec"]["version"]
    kind = crd["spec"]["names"]["kind"]
    return GVK(kind=kind, api_version="%s/%s" % (group, version))


class OpenshiftSpecsImpl(specs.Openshift):
    cluster_operators = resource(kind="ClusterOperator", api_version="config.openshift.io/v1")
    crds = resource(kind="CustomResourceDefinition", api_version="apiextensions.k8s.io/v1beta1")
    crs = foreach_resource(crds, cr_gvk)
    machine_configs = resource(kind="MachineConfig", api_version="machineconfiguration.openshift.io/v1")
    machines = resource(kind="Machine", api_version="machine.openshift.io/v1beta1")
    namespaces = resource(kind="Namespace")
    nodes = resource(kind="Node")
    pods = resource(kind="Pod", field_selector="status.phase!=Succeeded")
    pvcs = resource(kind="PersistentVolumeClaim")
    storage_classes = resource(kind="StorageClass")

    @datasource(namespaces)
    def machine_id(broker):
        doc = json.loads(broker[OpenshiftSpecsImpl.namespaces].content)
        v = next(o["metadata"]["uid"] for o in doc["items"] if o["metadata"]["name"] == "kube-system")
        return DatasourceProvider(content=v, relative_path="/etc/insights-client/machine-id")



import pytest
from insights.parsers.installed_rpms import InstalledRpms, InstalledRpm, pad_version
from insights.tests import context_wrap


RPMS_PACKAGE = '''
openjpeg-libs-1.3-9.el6_3.x86_64
openldap-2.4.23-31.el6.x86_64
openobex-1.4-7.el6.x86_64
openssh-server-5.3p1-104.el6.x86_64
openssh-askpass-5.3p1-84.1.el6.x86_64
openssl-1.0.0-27.el6.x86_64
'''.strip()

RPMS_PACKAGE_WITH_GARBAGE = '''
openjpeg-libs-1.3-9.el6_3.x86_64
openldap-2.4.23-31.el6.x86_64
openobex-1.4-7.el6.x86_64
openssh-server#$%^5.3p1$%^104.el6.x86_64
openssh-askpass-5.3p1-84.1.el6.x86_64
openssl-1.0.0-27.el6.x86_64
'''.strip()

RPMS_LINE = '''
COMMAND> rpm xxxxx
BESAgent-9.2.5.130-rhe5.x86_64                              Wed Jan 27 09:18:52 2016	1453904332	IBM Corp.	rhel5x64bs	(none)	(none)
ConsoleKit-0.4.1-3.el6.x86_64                               Wed Sep 17 11:34:25 2014
kernel-2.6.32-573.el6.x86_64                                Wed May 18 14:17:17 2016
zisofs-tools-1.0.6-3.2.2.x86_64                             Wed May 18 14:15:26 2016
tftp-server-5.2-12.el7.x86_64
yum-3.2.29-69.el6.noarch                                    Wed May 18 14:16:21 2016	1410968065	Red Hat, Inc.	hs20-bc2-4.build.redhat.com	8902150305004c6955c9199e2f91fd431d51010853c80ffb05f6916ea5f2980314b1766656aeb544777db76296173ffc16708d725e7b691e4ad9aef3dbe9c544a5d33264c9b4bf36464bec32ae6960be72c0175d710333f7aa52e24fc774d1c8809c2730d381593214b51abf7a455354e56993eac5a536fbdda1f1530ca8a9b5816bb47f0a5cf60081462aef48c8a68c761cd15d01f53eabc10aa4b90f47fac18ee8094d1613195277120a85efc6a0c9e2dbbb619f520c59ee74a7be84b2ae4177ece2e10f18617bfb926eade8537993f16fbd28e4a95e3bf7acf381b847171e0a03c241c5c20b5cfb021f69903b9afa4ce40d2bf17b5b439b1014bb974becca815b268f209833dce8cdd4052020aee680a56d1eff7214fb36bbcd35a0674374df2a64e3c3f13c10f23c7d33d035f3a7c7525e7037868ef86681ce0f41d9376f0ac4b1176a939752e1b63cb5a49e9004ee6b9d797ee16dd7b5b97496b3c1f5c0fd792e28117887af78026eb8422077e27d32fb0dba7025d870e8db7fa4e6abfb221abe9b1997ab808c07220329e0ec7863dfcf0c1c1931c4f0061015b902a6d65e59ca8a9252dcf4eeeac3cdfbbf4664a356d6f05e031bcd9cc3223d10425a03bacbe2f49d3f54bd2288f64c9812a5c255e6c11a4dd46255c3a9f2bed34b61bbd849bcf1f007d5f4e3f6e60064b372e0ac6ab301163393adb10d9aa47fe6be211b3576ff37da7e12e2285358267495ac48a437d4eefb3213	RSA/8, Mon Aug 16 11:14:17 2010, Key ID 199e2f91fd431d51
'''.strip()

RPM_MANIFEST = '''
gpg-pubkey-0608b895-4bd22942                  Sat Aug 29 19:29:30 2015
capacity-osms-0.0.1-1.noarch                  Sat Aug 29 19:17:55 2015
HPOvGlanc-11.14.014-1.x86_64                  Sat Aug 29 19:17:01 2015
HPOvPerfAgt-11.14.014-1.x86_64                Sat Aug 29 19:16:39 2015
HPOvPerfMI-11.14.014-1.x86_64                 Sat Aug 29 19:16:32 2015
HPOvAgtLc-11.14.014-1.x86_64                  Sat Aug 29 19:16:02 2015
vmware-tools-8.3.19-1310361.el6.x86_64        Sat Aug 29 19:10:11 2015
vmware-open-vm-tools-xorg-utilities-8.3.19-1310361.el6.x86_64 Sat Aug 29 19:10:11 2015
vmware-open-vm-tools-8.3.19-1310361.el6.x86_64 Sat Aug 29 19:10:11 2015
vmware-tools-nox-8.3.19-1310361.el6.x86_64    Sat Aug 29 19:10:10 2015
vmware-tools-common-8.3.19-1310361.el6.x86_64 Sat Aug 29 19:10:10 2015
vmware-open-vm-tools-xorg-drv-display-11.0.1.0-0.1310361.el6.x86_64 Sat Aug 29 19:10:10 2015
vmware-open-vm-tools-nox-8.3.19-1310361.el6.x86_64 Sat Aug 29 19:10:10 2015
vmware-open-vm-tools-common-8.3.19-1310361.el6.x86_64 Sat Aug 29 19:10:09 2015
vmware-open-vm-tools-xorg-drv-mouse-12.6.7.0-0.1310361.el6.x86_64 Sat Aug 29 19:10:08 2015
'''

RPMS_JSON = '''
{"name": "util-linux","version": "2.23.2","epoch": "(none)","release": "26.el7_2.2","arch": "x86_64","installtime": "Fri 24 Jun 2016 04:17:58 PM EDT","buildtime": "1458159298","rsaheader": "RSA/SHA256, Sun 20 Mar 2016 10:00:45 PM EDT, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "util-linux-2.23.2-26.el7_2.2.src.rpm"}
{"name": "libestr","version": "0.1.9","epoch": "(none)","release": "2.el7","arch": "x86_64","installtime": "Fri 06 May 2016 03:53:26 PM EDT","buildtime": "1390734694","rsaheader": "RSA/SHA256, Tue 01 Apr 2014 04:49:20 PM EDT, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "libestr-0.1.9-2.el7.src.rpm"}
{"name": "log4j","version": "1.2.17","epoch": "0","release": "15.el7","arch": "noarch","installtime": "Thu 02 Jun 2016 05:10:29 PM EDT","buildtime": "1388247429","rsaheader": "RSA/SHA256, Wed 02 Apr 2014 11:25:59 AM EDT, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "log4j-1.2.17-15.el7.src.rpm"}
{"name": "kbd-misc","version": "1.15.5","epoch": "(none)","release": "11.el7","arch": "noarch","installtime": "Fri 06 May 2016 03:52:06 PM EDT","buildtime": "1412004323","rsaheader": "RSA/SHA256, Tue 16 Dec 2014 10:02:15 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "kbd-1.15.5-11.el7.src.rpm"}
{"name": "grub2-tools","version": "2.02","epoch": "1","release": "0.34.el7_2","arch": "x86_64","installtime": "Fri 24 Jun 2016 04:18:01 PM EDT","buildtime": "1450199819","rsaheader": "RSA/SHA256, Wed 23 Dec 2015 04:22:27 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "grub2-2.02-0.34.el7_2.src.rpm"}
{"name": "kbd-legacy","version": "1.15.5","epoch": "(none)","release": "11.el7","arch": "noarch","installtime": "Fri 06 May 2016 03:53:32 PM EDT","buildtime": "1412004323","rsaheader": "RSA/SHA256, Tue 16 Dec 2014 10:02:14 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "kbd-1.15.5-11.el7.src.rpm"}
{"name": "jboss-servlet-3.0-api","version": "1.0.1","epoch": "(none)","release": "9.el7","arch": "noarch","installtime": "Thu 02 Jun 2016 05:10:30 PM EDT","buildtime": "1388211302","rsaheader": "RSA/SHA256, Tue 01 Apr 2014 02:51:30 PM EDT, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "jboss-servlet-3.0-api-1.0.1-9.el7.src.rpm"}
{"name": "bash","version": "4.2.46","epoch": "(none)","release": "19.el7","arch": "x86_64","installtime": "Fri 06 May 2016 03:52:13 PM EDT","buildtime": "1436354006","rsaheader": "RSA/SHA256, Wed 07 Oct 2015 01:14:10 PM EDT, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "bash-4.2.46-19.el7.src.rpm"}
{"name": "ca-certificates","version": "2015.2.6","epoch": "(none)","release": "70.1.el7_2","arch": "noarch","installtime": "Fri 24 Jun 2016 04:18:04 PM EDT","buildtime": "1453976868","rsaheader": "RSA/SHA256, Tue 02 Feb 2016 09:45:04 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "ca-certificates-2015.2.6-70.1.el7_2.src.rpm"}
{"name": "jline","version": "1.0","epoch": "(none)","release": "8.el7","arch": "noarch","installtime": "Thu 02 Jun 2016 05:10:32 PM EDT","buildtime": "1388212830","rsaheader": "RSA/SHA256, Tue 01 Apr 2014 02:54:16 PM EDT, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "jline-1.0-8.el7.src.rpm"}
{"name": "libteam","version": "1.17","epoch": "(none)","release": "6.el7_2","arch": "x86_64","installtime": "Fri 24 Jun 2016 04:18:17 PM EDT","buildtime": "1454604485","rsaheader": "RSA/SHA256, Wed 17 Feb 2016 02:25:16 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "libteam-1.17-6.el7_2.src.rpm"}
{"name": "crash","epoch":"(none)","version":"7.1.0","release":"8.el6","arch":"x86_64","installtime":"Fri Jul 13 06:53:28 2018","buildtime":"1524061059","vendor":"Red Hat, Inc.","buildhost":"x86-032.build.eng.bos.redhat.com","sigpgp":"RSA/8, Wed Apr 18 10:40:59 2018, Key ID 199e2f91fd431d51"}
{"name": "xorg-x11-drv-vmmouse","epoch":"(none)","version":"13.1.0","release":"1.el6","arch":"x86_64","installtime":"Thu Aug  4 12:23:32 2016","buildtime":"1447274489","vendor":"Red Hat, Inc.","buildhost":"x86-028.build.eng.bos.redhat.com","sigpgp":"RSA/8, Mon Apr 4 11:35:36 2016, Key ID 199e2f91fd431d51"}
{"name": "libnl","epoch":"(none)","version":"1.1.4","release":"2.el6","arch":"x86_64","installtime":"Mon Jun 16 13:21:21 2014","buildtime":"1378459378","vendor":"Red Hat, Inc.","buildhost":"x86-007.build.bos.redhat.com","sigpgp":"RSA/8, Mon Sep 23 07:25:47 2013, Key ID 199e2f91fd431d51"}
'''.strip()

RPMS_MULTIPLE = '''
yum-3.4.3-132.el7.noarch
yum-3.4.2-132.el7.noarch
'''

RPMS_MULTIPLE_KERNEL = '''
kernel-3.10.0-327.el7.x86_64
kernel-3.10.0-327.36.1.el7.x86_64
kernel-devel-3.10.0-327.el7.x86_64
kernel-devel-3.10.0-327.36.1.el7.x86_64
'''

ERROR_DB = '''
error: rpmdbNextIterator: skipping h#     753 Header V3 DSA signature: BAD, key ID db42a6
yum-security-1.1.16-21.el5.noarch
'''.strip()

ERROR_DB_NO_PKG = """
error: rpmdb: BDB0113 Thread/process 20263/140251984590912 failed: BDB1507 Thread died in Berkeley DB library
error: db5 error(-30973) from dbenv->failchk: BDB0087 DB_RUNRECOVERY: Fatal error, run database recovery
error: cannot open Packages index using db5 -  (-30973)
error: cannot open Packages database in /var/lib/rpm
error: rpmdb: BDB0113 Thread/process 20263/140251984590912 failed: BDB1507 Thread died in Berkeley DB library
error: db5 error(-30973) from dbenv->failchk: BDB0087 DB_RUNRECOVERY: Fatal error, run database recovery
error: cannot open Packages database in /var/lib/rpm
""".strip()

ORACLEASM_RPMS = '''
oracleasm-2.6.18-164.el5-2.0.5-1.el5.x86_64
oracleasmlib-2.0.4-1.el5.x86_64
oracleasm-support-2.1.3-1.el5.x86_64
oracle-validated-1.0.0-18.el5.x86_64
'''.strip()

NON_ORACLEASM_RPMS = '''
oracleasmlib-2.0.4-1.el5.x86_64
oracleasm-support-2.1.3-1.el5.x86_64
oracle-validated-1.0.0-18.el5.x86_64
'''.strip()

RHV_HYPERVISOR_RPMS = '''
vdsm-jsonrpc-4.18.15.3-1.el7ev.noarch
vdsm-hook-vmfex-dev-4.18.15.3-1.el7ev.noarch
vdsm-cli-4.18.15.3-1.el7ev.noarch
vdsm-4.18.15.3-1.el7ev.x86_64
'''.strip()

RPMS_PACKAGE_WITH_UNICODE = u'''
openjpeg-libs-1.3-9.el6_3.x86_64
openldap-2.4.23-31.el6.x86_64
openobex\u018e-1.4-7.el6.x86_64
openssh-server-5.3p1-104.el6.x86_64
openssh-askpass-5.3p1-84.1.el6.x86_64
openssl-1.0.0-27.el6.x86_64
'''.strip()

RPMS_GPG_PUBKEY = '''
{"name":"gpg-pubkey","epoch":"(none)","version":"2fa658e0","release":"45700c69","arch":"(none)","installtime":"Thu Apr 25 08:21:00 2019","buildtime":"1556194860","vendor":"(none)","buildhost":"localhost","sigpgp":"(none)"}
'''.strip()

RPMS_VMAAS = '''
compat-libstdc++-296-2.96-138
compat-libstdc++-296-2.96-144.el6
compat-libstdc++-33-3.2.3-47.3
compat-libstdc++-33-3.2.3-61
compat-libstdc++-33-3.2.3-69.el6
compat-libstdc++-33-3.2.3-72.el7
flash-plugin-11.2.202.635-release
flash-plugin-31.0.0.148-1.el6_10
flash-plugin-32.0.0.156-release
flash-plugin-32.0.0.171-1.el6_10
glibc-2.12-1.149.el6_6.5
glibc-2.5-81
infinipath-psm-3.0.1-115.1015_open.2.el6
infinipath-psm-3.3-1.7.git05f6f14_open.el6
infinipath-psm-3.3-19_g67c0807_open.el6
kmod-fnic-1.5.0.45-1.el6
kmod-fnic-1.6.0.12b-rhel6u6.el6
krb5-libs-1.10.3-42z1.el6_7
krb5-libs-1.10.3-65.el6
libaio-devel-0.3.106-5
libaio-devel-0.3.107-10.el6
mdadm-4.0-13.el7
mdadm-4.1-rc1_2.el7
nagios-plugins-disk-2.2.1-16.20180725git3429dad.el7
nagios-plugins-disk-2.2.1-9git5c7eb5b9.el7
TaniumClient-6.0.314.1321-1
TaniumClient-6.0.314.1579_1.rhe7-1
TaniumClient-7.2.314.2962_1.oel6-1
TaniumClient-7.2.314.2962-1.rhe6
TaniumClient-7.2.314.3211-1.rhe7
xymon-4.3.11-11.1
xymon-4.3.25-1.el6
xz-5.1.2-12alpha.el7
xz-5.2.2-1.el7
xz-libs-5.1.2-12alpha.el7
xz-libs-5.2.2-1.el7
'''.strip()


def test_from_package():
    rpms = InstalledRpms(context_wrap(RPMS_PACKAGE))
    assert not rpms.is_hypervisor
    pkg_rpm = rpms.packages['openssh-server'][0]
    rpm = InstalledRpm.from_package(pkg_rpm.package)
    assert rpm.package == 'openssh-server-5.3p1-104.el6'
    assert pkg_rpm.package == 'openssh-server-5.3p1-104.el6'
    assert rpm == pkg_rpm
    assert rpm.epoch == '0'

    rpm = InstalledRpm.from_package('bash-1:4.1.2-12.el7.x86_64')
    assert rpm.epoch == '1'


def test_from_line():
    rpms = InstalledRpms(context_wrap(RPMS_LINE))
    assert rpms.get_max("ConsoleKit").arch == 'x86_64'
    assert rpms.get_max("kernel").version == '2.6.32'
    assert rpms.get_max("yum").release == '69.el6'
    assert rpms.get_max("tftp-server").version == '5.2'
    assert rpms.get_max("yum").package == "yum-3.2.29-69.el6"
    assert rpms.get_max("tftp-server").redhat_signed is None
    assert rpms.get_max("yum").redhat_signed
    assert rpms.corrupt is False

    assert rpms.newest('BESAgent').vendor == 'IBM Corp.'
    assert rpms.newest('yum').vendor == 'Red Hat, Inc.'
    assert rpms.newest('kernel').vendor is None


def test_from_json():
    rpms = InstalledRpms(context_wrap(RPMS_JSON))
    assert isinstance(rpms.get_max("log4j").source, InstalledRpm)
    assert len(rpms.packages) == len(RPMS_JSON.splitlines())
    assert rpms.get_max("log4j").source.name == "log4j"
    assert rpms.get_max("util-linux").epoch == '0'
    assert rpms.get_max("jboss-servlet-3.0-api").redhat_signed

    assert rpms.newest('libnl').vendor == 'Red Hat, Inc.'
    assert rpms.newest('log4j').vendor is None


def test_garbage():
    rpms = InstalledRpms(context_wrap(RPMS_PACKAGE_WITH_GARBAGE))
    assert 'openssh-server' not in rpms


def test_corrupt_db():
    rpms = InstalledRpms(context_wrap(ERROR_DB))
    assert rpms.corrupt is True
    assert "yum-security" in rpms.packages
    assert "yum-security" in rpms

    rpms = InstalledRpms(context_wrap(ERROR_DB_NO_PKG))
    assert rpms.corrupt is True
    assert not rpms.packages
    assert "kernel" not in rpms
    assert "kernel" not in rpms.packages
    assert rpms.newest("kernel") is None
    assert rpms.oldest("kernel") is None


def test_rpm_manifest():
    rpms = InstalledRpms(context_wrap(RPM_MANIFEST))
    assert 'gpg-pubkey' in rpms
    assert rpms.packages['vmware-tools'][0].package == \
        'vmware-tools-8.3.19-1310361.el6'
    assert rpms.packages['vmware-tools'][0].installtime == \
        'Sat Aug 29 19:10:11 2015'


def test_package_property_aliases():
    rpms = InstalledRpms(context_wrap(RPMS_JSON))
    rpm = rpms.get_max("grub2-tools")
    assert rpm.package == "grub2-tools-2.02-0.34.el7_2"
    assert rpm.nvr == "grub2-tools-2.02-0.34.el7_2"
    assert rpm.nvra == "grub2-tools-2.02-0.34.el7_2.x86_64"


def test_max_min():
    rpms = InstalledRpms(context_wrap(RPMS_MULTIPLE))
    assert rpms.get_min('yum').package == 'yum-3.4.2-132.el7'
    assert rpms.get_max('yum').package == 'yum-3.4.3-132.el7'


def test_max_min_not_found():
    rpms = InstalledRpms(context_wrap(RPMS_MULTIPLE_KERNEL))
    assert rpms.get_min('abc') is None
    assert rpms.get_max('abc') is None


def test_max_min_kernel():
    rpms = InstalledRpms(context_wrap(RPMS_MULTIPLE_KERNEL))
    assert rpms.get_min('kernel').package == 'kernel-3.10.0-327.el7'
    assert rpms.get_max('kernel').package == 'kernel-3.10.0-327.36.1.el7'
    assert rpms.get_min('kernel-devel').package == 'kernel-devel-3.10.0-327.el7'
    assert rpms.get_max('kernel-devel').package == 'kernel-devel-3.10.0-327.36.1.el7'


def test_release_compare():
    rpm1 = InstalledRpm.from_package('kernel-rt-debug-3.10.0-327.rt56.204.el7_2.1')
    rpm2 = InstalledRpm.from_package('kernel-rt-debug-3.10.0-327.rt56.204.el7_2.2')
    rpm3 = InstalledRpm.from_package('kernel-3.10.0-327.10.1.el7')
    rpm4 = InstalledRpm.from_package('kernel-3.10.0-327.el7')
    rpm5 = InstalledRpm.from_package('kernel-3.10.0-327.el7_1')
    rpm6 = InstalledRpm.from_package('kernel-3.10.0-327')
    rpm7 = InstalledRpm.from_package('kernel-3.10.0-327.1')
    rpm8 = InstalledRpm.from_package('kernel-3.10.0-327.x86_64')
    rpm9 = InstalledRpm.from_package('kernel-3.10.0-327.1.x86_64')
    with pytest.raises(ValueError) as ve:
        rpm1 > rpm7
    assert "differing names" in str(ve)
    with pytest.raises(ValueError) as ve:
        rpm1 <= rpm7
    assert "differing names" in str(ve)
    with pytest.raises(ValueError) as ve:
        rpm1 != rpm7
    assert "differing names" in str(ve)

    assert rpm1 < rpm2
    assert rpm1 != rpm2
    assert rpm3 > rpm4
    assert rpm5 > rpm4
    assert rpm5 >= rpm4
    assert not (rpm5 <= rpm4)
    assert rpm6 < rpm7
    assert rpm8 < rpm9
    assert rpm6 < rpm9
    assert rpm6 <= rpm9
    assert rpm6 == rpm8
    assert rpm6 <= rpm8
    assert rpm6 >= rpm8
    assert not (rpm7 != rpm9)


def test_version_compare():
    d1 = {
        "name": "bash",
        "version": "4.2.46",
        "epoch": "0",
        "release": "19.el7",
        "arch": "x86_64",
        "installtime": "Fri 06 May 2016 03:52:13 PM EDT",
        "buildtime": "1436354006",
        "rsaheader": "RSA/SHA256,Wed 07 Oct 2015 01:14:10 PM EDT,Key ID 199e2f91fd431d51",
        "dsaheader": "(none)",
        "srpm": "bash-4.2.46-19.el7.src.rpm"
    }

    d2 = d1.copy()
    d2["epoch"] = "1"

    rpm1, rpm2 = InstalledRpm(d1), InstalledRpm(d2)

    assert rpm2 > rpm1
    assert rpm2.source > rpm1.source
    assert rpm1 == rpm1
    assert not rpm1 < rpm1
    assert rpm2 != rpm1
    assert rpm2 >= rpm1
    assert rpm1 <= rpm2

    assert not rpm1 < None
    assert not rpm1 <= None
    assert not rpm1 > None
    assert not rpm1 >= None
    assert not rpm1.__eq__(None)

    assert not [rpm1, rpm2] > rpm2
    assert not rpm1 > ""
    assert not rpm1 > 4
    assert not rpm1 > {}
    assert not rpm1 > set()
    assert not rpm1 > []


def test_formatting():
    rpm = InstalledRpm.from_package('kernel-3.10.0-327.el7.x86_64')
    assert str(rpm) == '0:kernel-3.10.0-327.el7'
    assert repr(rpm) == '0:kernel-3.10.0-327.el7'


def test_exceptions():
    rpm1 = InstalledRpm.from_package('kernel-3.10.0-327.el7.x86_64')
    rpm2 = InstalledRpm.from_package('plevel-3.10.0-327.el7.x86_64')
    with pytest.raises(ValueError):
        assert rpm1 > rpm2


def test_different_arch_sep():
    rpm1 = InstalledRpm.from_package('yum-3.4.3-132.el7.noarch')
    rpm2 = InstalledRpm.from_package('yum-3.4.3-132.el7-noarch')
    assert rpm1 == rpm2
    assert rpm1.arch == rpm2.arch
    assert rpm1['arch'] == rpm2.arch


def test_no_suffixes():
    # make sure 'in' really means 'has a package that starts with'
    rpms = InstalledRpms(context_wrap(RPMS_PACKAGE))
    assert 'openssh-askpass' in rpms
    assert 'askpass' not in rpms
    assert 'openobex' in rpms
    assert 'penobex' not in rpms


def test_oracleasmrpms():
    # Oracle RPMs have a weird format - fix that in the read process
    ora_rpms = InstalledRpms(context_wrap(ORACLEASM_RPMS))
    assert ora_rpms is not None
    assert 'oracleasm' in ora_rpms.packages
    assert ora_rpms.get_max('oracleasm').version == '2.6.18-164.el5-2.0.5'
    assert ora_rpms.get_max('oracleasm').release == '1.el5'
    assert 'oracleasmlib' in ora_rpms.packages
    assert ora_rpms.get_max('oracleasmlib').version == '2.0.4'
    assert ora_rpms.get_max('oracleasmlib').release == '1.el5'
    assert 'oracleasm-support' in ora_rpms.packages
    assert ora_rpms.get_max('oracleasm-support').version == '2.1.3'
    assert ora_rpms.get_max('oracleasm-support').release == '1.el5'


def test_is_hypervisor():
    rpms = InstalledRpms(context_wrap(RHV_HYPERVISOR_RPMS))
    assert "vdsm" in rpms.packages
    assert rpms.is_hypervisor


def test_unicode_char_in_rpms():
    rpms = InstalledRpms(context_wrap(RPMS_PACKAGE_WITH_UNICODE))
    assert u"openobex\u018e" in rpms.packages
    rpm = rpms.get_max(u'openobex\u018e')
    assert rpm.package == u'openobex\u018e-1.4-7.el6'


def test_pad_version_uneven_sections():
    assert pad_version('1.el7', '1.el7_4.ngx') == ([1, 'el', 7, 0, ''], [1, 'el', 7, 4, 'ngx'])


def test_epoch():
    rpms = InstalledRpms(context_wrap(RPMS_PACKAGE))
    # no epoch
    rpm = rpms.get_max('openldap')
    assert rpm.package_with_epoch == 'openldap-0:2.4.23-31.el6'
    assert rpm.nevra == 'openldap-0:2.4.23-31.el6.x86_64'

    rpms = InstalledRpms(context_wrap(RPMS_JSON))
    # epoch 0
    rpm = rpms.get_max('log4j')
    assert rpm.package_with_epoch == 'log4j-0:1.2.17-15.el7'
    assert rpm.nevra == 'log4j-0:1.2.17-15.el7.noarch'
    # epoch (none)
    rpm = rpms.get_max('kbd-misc')
    assert rpm.package_with_epoch == 'kbd-misc-0:1.15.5-11.el7'
    assert rpm.nevra == 'kbd-misc-0:1.15.5-11.el7.noarch'
    # epoch 1
    rpm = rpms.get_max('grub2-tools')
    assert rpm.package_with_epoch == 'grub2-tools-1:2.02-0.34.el7_2'
    assert rpm.nevra == 'grub2-tools-1:2.02-0.34.el7_2.x86_64'


def test_rpm_object_hashing():
    # Class InstalledRpm implements for hashing function __hash__().
    # This allows to use objects InstalledRpm in set() and dict().

    # just NVR
    rpm_yum1 = InstalledRpm.from_package('yum-3.4.3-132.el7')
    assert rpm_yum1.arch is None
    rpm_yum2 = InstalledRpm.from_package('yum-3.4.3-132.el7')
    assert rpm_yum2.arch is None
    rpm_kernel = InstalledRpm.from_package('kernel-3.10.0-327')
    assert rpm_kernel.arch is None
    assert rpm_yum1.__hash__() == rpm_yum2.__hash__()
    assert rpm_yum1.__hash__() != rpm_kernel.__hash__()
    assert rpm_yum2.__hash__() != rpm_kernel.__hash__()

    # NVRA
    rpm1 = InstalledRpm.from_package('yum-3.4.3-132.el7.i686')
    assert rpm1.arch == 'i686'
    rpm2 = InstalledRpm.from_package('yum-3.4.3-132.el7.i686')
    assert rpm2.arch == 'i686'
    rpm3 = InstalledRpm.from_package('kernel-3.10.0-327.x86_64')
    assert rpm3.arch == 'x86_64'
    assert rpm1.__hash__() == rpm2.__hash__()
    assert rpm1.__hash__() != rpm3.__hash__()
    assert rpm2.__hash__() != rpm3.__hash__()

    # NVR and NVRA
    rpm1 = InstalledRpm.from_package('yum-3.4.3-132.el7')
    assert rpm1.arch is None
    rpm2 = InstalledRpm.from_package('yum-3.4.3-132.el7.ppc64le')
    assert rpm2.arch == 'ppc64le'
    assert rpm1.__hash__() != rpm2.__hash__()


def test_gpgkey():
    rpms = InstalledRpms(context_wrap(RPMS_GPG_PUBKEY))
    assert isinstance(rpms.get_max("gpg-pubkey"), InstalledRpm)
    assert len(rpms.packages) == 1
    assert rpms.get_max("gpg-pubkey").name == "gpg-pubkey"


def test_vmaas():
    rpms = InstalledRpms(context_wrap(RPMS_VMAAS))
    rpm = rpms.get_max("compat-libstdc++-296")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "2.96"
    assert rpm.release == "144.el6"

    rpm = rpms.get_max("compat-libstdc++-33")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "3.2.3"
    assert rpm.release == "72.el7"

    rpm = rpms.get_max("flash-plugin")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "32.0.0.171"
    assert rpm.release == "1.el6_10"

    rpm = rpms.get_max("glibc")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "2.12"
    assert rpm.release == "1.149.el6_6.5"

    rpm = rpms.get_max("infinipath-psm")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "3.3"
    assert rpm.release == "19_g67c0807_open.el6"

    rpm = rpms.get_max("kmod-fnic")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "1.6.0.12b"
    assert rpm.release == "rhel6u6.el6"

    rpm = rpms.get_max("krb5-libs")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "1.10.3"
    assert rpm.release == "65.el6"

    rpm = rpms.get_max("libaio-devel")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "0.3.107"
    assert rpm.release == "10.el6"

    rpm = rpms.get_max("mdadm")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "4.1"
    assert rpm.release == "rc1_2.el7"

    rpm = rpms.get_max("nagios-plugins-disk")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "2.2.1"
    assert rpm.release == "16.20180725git3429dad.el7"

    rpm = rpms.get_max("TaniumClient")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "7.2.314.3211"
    assert rpm.release == "1.rhe7"

    rpm = rpms.get_max("xymon")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "4.3.25"
    assert rpm.release == "1.el6"

    rpm = rpms.get_max("xz")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "5.2.2"
    assert rpm.release == "1.el7"

    rpm = rpms.get_max("xz-libs")
    assert isinstance(rpm, InstalledRpm)
    assert rpm.version == "5.2.2"
    assert rpm.release == "1.el7"



from glob import glob
from insights.client.archive import InsightsArchive
from insights.client.connection import InsightsConnection
from insights.client.constants import InsightsConstants as constants
from insights.client.utilities import determine_hostname
from logging import getLogger
from platform import linux_distribution
from re import findall
from sys import exit
from insights.util.subproc import call
import os
import six

NONCOMPLIANT_STATUS = 2
COMPLIANCE_CONTENT_TYPE = 'application/vnd.redhat.compliance.something+tgz'
POLICY_FILE_LOCATION = '/usr/share/xml/scap/ssg/content/'
REQUIRED_PACKAGES = ['scap-security-guide', 'openscap-scanner', 'openscap']
logger = getLogger(__name__)


class ComplianceClient:
    def __init__(self, config):
        self.config = config
        self.conn = InsightsConnection(config)
        self.hostname = determine_hostname()
        self.archive = InsightsArchive(config)

    def oscap_scan(self):
        self._assert_oscap_rpms_exist()
        policies = self.get_policies()
        if not policies:
            logger.error("System is not associated with any profiles. Assign profiles by either uploading a SCAP scan or using the compliance web UI.\n")
            exit(constants.sig_kill_bad)
        for policy in policies:
            self.run_scan(
                policy['attributes']['ref_id'],
                self.find_scap_policy(policy['attributes']['ref_id']),
                '/var/tmp/oscap_results-{0}.xml'.format(policy['attributes']['ref_id']),
                tailoring_file_path=self.download_tailoring_file(policy)
            )

        return self.archive.create_tar_file(), COMPLIANCE_CONTENT_TYPE

    def download_tailoring_file(self, policy):
        if 'tailored' not in policy['attributes'] or policy['attributes']['tailored'] is False:
            return None

        # Download tailoring file to pass as argument to run_scan
        logger.debug(
            "Policy {0} is a tailored policy. Starting tailoring file download...".format(policy['attributes']['ref_id'])
        )
        tailoring_file_path = "/var/tmp/oscap_tailoring_file-{0}.xml".format(policy['attributes']['ref_id'])
        response = self.conn.session.get(
            "https://{0}/compliance/profiles/{1}/tailoring_file".format(self.config.base_url, policy['id'])
        )
        logger.debug("Response code: {0}".format(response.status_code))
        if response.content is None:
            logger.info("Problem downloading tailoring file for {0} to {1}".format(policy['attributes']['ref_id'], tailoring_file_path))
            return None

        with open(tailoring_file_path, mode="w+b") as f:
            f.write(response.content)
            logger.info("Saved tailoring file for {0} to {1}".format(policy['attributes']['ref_id'], tailoring_file_path))

        logger.debug("Policy {0} tailoring file download finished".format(policy['attributes']['ref_id']))

        return tailoring_file_path

    # TODO: Not a typo! This endpoint gives OSCAP policies, not profiles
    # We need to update compliance-backend to fix this
    def get_policies(self):
        response = self.conn.session.get("https://{0}/compliance/profiles".format(self.config.base_url),
                                         params={'search': 'system_names={0} external=false canonical=false'.format(self.hostname)})
        logger.debug("Content of the response: {0} - {1}".format(response,
                                                                 response.json()))
        if response.status_code == 200:
            return (response.json().get('data') or [])
        else:
            return []

    def os_release(self):
        _, version, _ = linux_distribution()
        return findall("^[6-8]", version)[0]

    def profile_files(self):
        return glob("{0}*rhel{1}*.xml".format(POLICY_FILE_LOCATION, self.os_release()))

    def find_scap_policy(self, profile_ref_id):
        grepcmd = 'grep ' + profile_ref_id + ' ' + ' '.join(self.profile_files())
        if not six.PY3:
            grepcmd = grepcmd.encode()
        rc, grep = call(grepcmd, keep_rc=True)
        if rc:
            logger.error('XML profile file not found matching ref_id {0}\n{1}\n'.format(profile_ref_id, grep))
            return None
        filenames = findall('/usr/share/xml/scap/.+xml', grep)
        if not filenames:
            logger.error('No XML profile files found matching ref_id {0}\n{1}\n'.format(profile_ref_id, ' '.join(filenames)))
            exit(constants.sig_kill_bad)
        return filenames[0]

    def build_oscap_command(self, profile_ref_id, policy_xml, output_path, tailoring_file_path):
        command = 'oscap xccdf eval --profile ' + profile_ref_id
        if tailoring_file_path:
            command += ' --tailoring-file ' + tailoring_file_path
        command += ' --results ' + output_path + ' ' + policy_xml
        return command

    def run_scan(self, profile_ref_id, policy_xml, output_path, tailoring_file_path=None):
        if policy_xml is None:
            return
        logger.info('Running scan for {0}... this may take a while'.format(profile_ref_id))
        env = os.environ.copy()
        env.update({'TZ': 'UTC'})
        oscap_command = self.build_oscap_command(profile_ref_id, policy_xml, output_path, tailoring_file_path)
        if not six.PY3:
            oscap_command = oscap_command.encode()
        rc, oscap = call(oscap_command, keep_rc=True, env=env)
        if rc and rc != NONCOMPLIANT_STATUS:
            logger.error('Scan failed')
            logger.error(oscap)
            exit(constants.sig_kill_bad)
        else:
            self.archive.copy_file(output_path)

    def _assert_oscap_rpms_exist(self):
        rpmcmd = 'rpm -qa ' + ' '.join(REQUIRED_PACKAGES)
        if not six.PY3:
            rpmcmd = rpmcmd.encode()
        rc, rpm = call(rpmcmd, keep_rc=True)
        if rc:
            logger.error('Tried running rpm -qa but failed: {0}.\n'.format(rpm))
            exit(constants.sig_kill_bad)
        else:
            if len(rpm.strip().split('\n')) < len(REQUIRED_PACKAGES):
                logger.error('Missing required packages for compliance scanning. Please ensure the following packages are installed: {0}\n'.format(', '.join(REQUIRED_PACKAGES)))
                exit(constants.sig_kill_bad)



"""
SSSD_Config - file ``/etc/sssd/sssd.config``
============================================

SSSD's configuration file is in a standard 'ini' format.

The 'sssd' section will define one or more active domains, which are then
configured in the 'domain/{domain}' section of the configuration.  These
domains are then available via the 'domains' method, and the configuration
of a domain can be fetched as a dictionary using the 'domain_config' method.

Example:
    >>> sssd_conf = shared[SSSD_Config]
    >>> sssd_conf.getint('nss', 'reconnection_retries')
    3
    >>> sssd_conf.domains()
    ['example.com']
    >>> domain = sssd_conf.domain_config('example.com')
    >>> 'ldap_uri' in domain
    True
"""

from .. import IniConfigFile, parser
from insights.specs import Specs


@parser(Specs.sssd_config)
class SSSD_Config(IniConfigFile):
    """
    Parse the content of the ``/etc/sssd/sssd.config`` file.

    The 'sssd' section must always exist.  Within that, the 'domains'
    parameter is usually defined to give a comma-separated list of the
    domains that sssd is to manage.
    """

    @property
    def domains(self):
        """
        Returns the list of domains defined in the 'sssd' section.  This is
        used to refer to the domain-specific sections of the configuration.
        """
        if self.has_option('sssd', 'domains'):
            domains = self.get('sssd', 'domains')
            # Return a blank list if no domains listed.
            if not domains:
                return []
            return domains.split(',')
        else:
            return []

    def domain_config(self, domain):
        """
        Return the configuration dictionary for a specific domain, given as
        the raw name as listed in the 'domains' property of the sssd section.
        This then looks for the equivalent 'domain/{domain}' section of the
        config file.
        """
        full_domain = 'domain/' + domain
        if full_domain not in self:
            return {}
        return self.items(full_domain)



"""
DockerList - command ``/usr/bin/docker (images|ps)``
====================================================

Parse the output of command "docker_list_images" and "docker_list_containers",
which have very similar formats.

The header line is parsed and used as the names for the remaining columns.
All fields in both header and data are assumed to be separated by at least
three spaces.  This allows single spaces in values and headers, so headers
such as 'IMAGE ID' are captured as is.

If the header line and at least one data line are not found, no data is
stored.

Each row is stored as a dictionary, keyed on the header fields.  The data is
available in two formats:

* The old format is a list of row dictionaries.
* The new format stores each dictionary in a dictionary keyed on the value of
  a given field, given by the subclass.

"""
from insights import CommandParser, parser
from insights.parsers import SkipException, parse_fixed_table
from insights.specs import Specs


class DockerList(CommandParser):
    """
    A general class for parsing tabular docker list information.  Parsing
    rules are:

    * The first line is the header line.
    * The other lines are data lines.
    * All fields line up vertically.
    * Fields are separated from each other by at least three spaces.
    * Some fields can contain nothing, and this is shown as spaces, so we
      need to catch that and turn it into None.

    Why not just use hard-coded fields and columns?  So that we can adapt to
    different output lists.

    Raises:
        NotImplementedError: If `key_field` or `attr_name` is not defined
        SkipException: If no data to parse
    """
    key_field = None
    heading_ignore = []
    attr_name = ''
    substitutions = []

    def parse_content(self, content):
        """
        Parse the lines given into a list of dictionaries for each row.  This
        is stored in the ``rows`` attribute.

        If the ``key_field`` property is set, use this to key a ``data``
        dictionary attribute.
        """
        if not (self.key_field and self.attr_name):
            raise NotImplementedError("'key_field' or 'attr_name' is not defined")

        self.rows = parse_fixed_table(content,
                                      heading_ignore=self.heading_ignore,
                                      header_substitute=self.substitutions)

        if not self.rows:
            raise SkipException('No data.')

        data = {}
        for row in self.rows:
            k = row.get(self.key_field)
            for sub in self.substitutions:
                row[sub[0]] = row.pop(sub[1]) if sub[1] in row else None
            if k is not None and k != '<none>':
                data[k] = row
        setattr(self, self.attr_name, data)


@parser(Specs.docker_list_images)
class DockerListImages(DockerList):
    """
    Handle the list of docker images using the DockerList parser class.

    Sample output of command ``docker images --all --no-trunc --digests``::

        REPOSITORY                           TAG                 DIGEST              IMAGE ID                                                           CREATED             VIRTUAL SIZE
        rhel7_imagemagick                    latest              <none>              882ab98aae5394aebe91fe6d8a4297fa0387c3cfd421b2d892bddf218ac373b2   4 days ago          785.4 MB
        rhel6_nss-softokn                    latest              <none>              dd87dad2c7841a19263ae2dc96d32c501ee84a92f56aed75bb67f57efe4e48b5   5 days ago          449.7 MB

    Attributes:
        rows (list): List of row dictionaries.
        images (dict): Dictionary keyed on the value of the "REPOSITORY" fileld

    Examples:
        >>> images.rows[0]['REPOSITORY']
        'rhel6_vsftpd'
        >>> images.rows[1]['VIRTUAL SIZE']
        '785.4 MB'
        >>> images.images['rhel6_vsftpd']['CREATED']
        '37 minutes ago'
    """
    key_field = 'REPOSITORY'
    heading_ignore = [key_field]
    attr_name = 'images'
    substitutions = [("IMAGE ID", "IMAGE_ID"), ("VIRTUAL SIZE", "VIRTUAL_SIZE")]

    @property
    def data(self):
        return self.images


@parser(Specs.docker_list_containers)
class DockerListContainers(DockerList):
    """
    Handle the list of docker images using the DockerList parser class.

    Sample output of command ``docker ps --all --no-trunc --size``::

        CONTAINER ID                                                       IMAGE                                                              COMMAND                                            CREATED             STATUS                        PORTS                  NAMES               SIZE
        95516ea08b565e37e2a4bca3333af40a240c368131b77276da8dec629b7fe102   bd8638c869ea40a9269d87e9af6741574562af9ee013e03ac2745fb5f59e2478   "/bin/sh -c 'yum install -y vsftpd-2.2.2-6.el6'"   51 minutes ago      Exited (137) 50 minutes ago                          tender_rosalind     4.751 MB (virtual 200.4 MB)
        03e2861336a76e29155836113ff6560cb70780c32f95062642993b2b3d0fc216   rhel7_httpd                                                        "/usr/sbin/httpd -DFOREGROUND"                     45 seconds ago      Up 37 seconds                 0.0.0.0:8080->80/tcp   angry_saha          796 B (virtual 669.2 MB)

    Attributes:
        rows (list): List of row dictionaries.
        containers(dict): Dictionary keyed on the value of the "NAMES" field

    Examples:
        >>> containers.rows[0]['STATUS']
        'Up 37 seconds'
        >>> containers.containers['tender_rosalind']['STATUS']
        'Exited (137) 18 hours ago'
    """
    key_field = 'NAMES'
    heading_ignore = ['CONTAINER']
    attr_name = 'containers'
    substitutions = [("CONTAINER ID", "CONTAINER_ID")]

    @property
    def data(self):
        return self.containers



"""
IPv6 - Check whether IPv6 is disabled
=====================================

This combiner reports whether the user has disabled IPv6 via one of the
many means available to do so. At present, only whether IPv6 is disabled on
the running system is reported; it provides no information regarding whether it
would continue to be after a reboot.

Per https://access.redhat.com/solutions/8709 , IPv6 may be disabled by

RHEL 7:
 * ``ipv6.disable=1`` Kernel command line argument
 * ``disable_ipv6`` option in ``sysctl``

RHEL 6:
 * ``option ipv6 disable=1`` in `modprobe.d`
 * ``install ipv6 /bin/true`` (fake install) in `modprobe.d`
 * ``disable_ipv6`` option in ``sysctl``

While they aren't tested explicitly, there are some means by which you can
attempt to disable IPv6 that are ineffective, such as setting
``blacklist ipv6`` in `modprobe.d`; those methods will yield no result from
this combiner.

The only requirement of this combiner is ``Uname``, but accurate detection
relies on information from combiners marked optional. If, for example, it's run
against a RHEL6 system without information from ``ModProbe``, it will miss any
of those disabling options and possibly return a false negative. For that
reason, this combiner shouldn't be relied on to state definitively that IPv6 is
enabled or disabled.

Examples:
    >>> from insights.tests import context_wrap
    >>> from insights.parsers.uname import Uname
    >>> from insights.parsers.sysctl import Sysctl
    >>> from insights.combiners.ipv6 import IPv6
    >>> my_uname = '''
    ...  Linux localhost.localdomain 3.10.0-514.10.2.el7.x86_64 #1 SMP Mon Feb 20 02:37:52 EST 2017 x86_64 x86_64 x86_64 GNU/Linux
    ... '''.strip()
    >>> my_sysctl = '''
    ... net.ipv6.conf.all.autoconf = 1
    ... net.ipv6.conf.all.dad_transmits = 1
    ... net.ipv6.conf.all.disable_ipv6 = 0
    ... net.ipv6.conf.all.force_mld_version = 0
    ... net.ipv6.conf.all.force_tllao = 0
    ... net.ipv6.conf.all.forwarding = 0
    ... '''.strip()
    >>> shared = {Uname: Uname(context_wrap(my_uname)), Sysctl: Sysctl(context_wrap(my_sysctl))}
    >>> my_ipv6 = IPv6({},shared)
    >>> my_ipv6.disabled()
    False
    >>> my_ipv6.disabled_by()
    set([])
    >>> my_sysctl = '''
    ... net.ipv6.conf.all.autoconf = 1
    ... net.ipv6.conf.all.dad_transmits = 1
    ... net.ipv6.conf.all.disable_ipv6 = 1
    ... net.ipv6.conf.all.force_mld_version = 0
    ... net.ipv6.conf.all.force_tllao = 0
    ... net.ipv6.conf.all.forwarding = 0
    ... '''.strip()
    >>> shared[Sysctl] = Sysctl(context_wrap(my_sysctl))
    >>> my_ipv6 = IPv6({},shared)
    >>> my_ipv6.disabled()
    True
    >>> my_ipv6.disabled_by()
    set(['sysctl'])

"""

from ..core.plugins import combiner
from ..parsers.modprobe import ModProbe
from ..parsers.lsmod import LsMod
from ..parsers.cmdline import CmdLine
from ..parsers.sysctl import Sysctl
from ..parsers.uname import Uname


RHEL_UNSUPPORTED_VERSION = 9999


@combiner(Uname, optional=[ModProbe, LsMod, CmdLine, Sysctl])
class IPv6(object):
    """A combiner which detects disabled IPv6 networking."""

    def __init__(self, uname, mod_probe, lsmod, cmdline, sysctl):
        self.disablers = set()

        if uname.rhel_release[0] == '7':
            self.rhelver = 7
        elif uname.rhel_release[0] == '6':
            self.rhelver = 6
        elif uname.rhel_release[0] == '5':
            self.rhelver = 5
        else:
            self.rhelver = RHEL_UNSUPPORTED_VERSION

        # ModProbe may be either a list or a ModProbe object, or
        # None if it wasn't provided. Make it safe to iterate over.
        self.modprobe = mod_probe or []
        if isinstance(self.modprobe, ModProbe):
            self.modprobe = [self.modprobe]

        # A dict of loaded modules if lsmod data was provided, or None,
        # because it's worth distinguishing whether the data was provided.
        self.lsmod = getattr(lsmod, 'data', None)

        # Nothing depends on command line data being provided, so default
        # to an empty dict
        self.cmdline = getattr(cmdline, 'data', {})

        # Likewise for sysctl data
        self.sysctl = getattr(sysctl, 'data', {})

        self._check_ipv6()

    def _check_ipv6(self):
        # Whether IPv6 is provided as a module or built in, it can be disabled
        # with the kernel command line
        if self.cmdline.get('ipv6.disable') == ['1']:
            self.disablers.add('cmdline')

        # IPv6 can also be disabled via sysctl. Though it may be disabled on a
        # per-interface basis, this combiner only reports if it's turned off
        # for the whole system.
        # Additionally, the disable_ipv6 option in the IPv6 kernel module sets
        # disable_ipv6 on each adapter, but for some reason not "all", which
        # may be missed by this combiner.
        # Also not reported is whether it's disabled in sysctl.conf and so
        # would persist between reboots.
        if self.sysctl.get('net.ipv6.conf.all.disable_ipv6') == '1':
            self.disablers.add('sysctl')

        # Pre-EL7 requires modprobe to state definitively whether the module
        # is disabled or not
        if (self.rhelver < 7 and len(self.modprobe) > 0):
            # Iterate over one or more sources of modprobe data
            for it in self.modprobe:
                # Whether or not the module is loaded, it may be disabled by
                # module options
                if it.get('options', {}).get('ipv6') == ['disable=1']:
                    self.disablers.add('modprobe_disable')

                # If the module is not currently loaded, a fake install will
                # prevent it from loading
                if self.lsmod is not None and 'ipv6' not in self.lsmod:
                    if it.get('install', {}).get('ipv6') == ['/bin/true']:
                        self.disablers.add('fake_install')

    def disabled(self):
        """Determine whether IPv6 has been disabled on this system.

        Returns:
            bool: True if a configuration that disables IPv6 was found.
        """
        return self.disablers != set([])

    def disabled_by(self):
        """Get the means by which IPv6 was disabled on this system.

        Returns:
            set: Zero or more of ``cmdline``, ``modprobe_disable``,
            ``fake_install``, or ``sysctl``, depending on which methods to
            disable IPv6 have been found.
        """
        return self.disablers



import pytest
import doctest
from insights.parsers import aws_instance_id
from insights.parsers.aws_instance_id import AWSInstanceIdDoc, AWSInstanceIdPkcs7
from insights.tests import context_wrap
from insights.parsers import SkipException, ParseException

AWS_CURL_ERROR = """
curl: (7) couldn't connect to host
"""

AWS_ID_DOC = """
{
    "devpayProductCodes" : null,
    "marketplaceProductCodes" : [ "1abc2defghijklm3nopqrs4tu" ],
    "availabilityZone" : "us-west-2b",
    "privateIp" : "10.158.112.84",
    "version" : "2017-09-30",
    "instanceId" : "i-1234567890abcdef0",
    "billingProducts" : [ "bp-6ba54002" ],
    "instanceType" : "t2.micro",
    "accountId" : "123456789012",
    "imageId" : "ami-5fb8c835",
    "pendingTime" : "2016-11-19T16:32:11Z",
    "architecture" : "x86_64",
    "kernelId" : null,
    "ramdiskId" : null,
    "region" : "us-west-2"
}"""

AWS_ID_DOC_CURL_STATS = """
% Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                 Dload  Upload   Total   Spent    Left  Speed

  0     0    0     0    0     0      0      0 --:--:-- --:--:-- --:--:--     0
100   483  100   483    0     0   607k      0 --:--:-- --:--:-- --:--:--  471k
{
    "devpayProductCodes" : null,
    "marketplaceProductCodes" : [ "1abc2defghijklm3nopqrs4tu" ],
    "availabilityZone" : "us-west-2b",
    "privateIp" : "10.158.112.84",
    "version" : "2017-09-30",
    "instanceId" : "i-1234567890abcdef0",
    "billingProducts" : [ "bp-6ba54002" ],
    "instanceType" : "t2.micro",
    "accountId" : "123456789012",
    "imageId" : "ami-5fb8c835",
    "pendingTime" : "2016-11-19T16:32:11Z",
    "architecture" : "x86_64",
    "kernelId" : null,
    "ramdiskId" : null,
    "region" : "us-west-2"
}"""

AWS_ID_DOC_ERROR = """
Invalid json
"""

AWS_NO_DOC = ""

AWS_ID_PKCS7 = """
MIICiTCCAfICCQD6m7oRw0uXOjANBgkqhkiG9w0BAQUFADCBiDELMAkGA1UEBhMC
VVMxCzAJBgNVBAgTAldBMRAwDgYDVQQHEwdTZWF0dGxlMQ8wDQYDVQQKEwZBbWF6
b24xFDASBgNVBAsTC0lBTSBDb25zb2xlMRIwEAYDVQQDEwlUZXN0Q2lsYWMxHzAd
BgkqhkiG9w0BCQEWEG5vb25lQGFtYXpvbi5jb20wHhcNMTEwNDI1MjA0NTIxWhcN
MTIwNDI0MjA0NTIxWjCBiDELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAldBMRAwDgYD
VQQHEwdTZWF0dGxlMQ8wDQYDVQQKEwZBbWF6b24xFDASBgNVBAsTC0lBTSBDb25z
b2xlMRIwEAYDVQQDEwlUZXN0Q2lsYWMxHzAdBgkqhkiG9w0BCQEWEG5vb25lQGFt
YXpvbi5jb20wgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBAMaK0dn+a4GmWIWJ
21uUSfwfEvySWtC2XADZ4nB+BLYgVIk60CpiwsZ3G93vUEIO3IyNoH/f0wYK8m9T
rDHudUZg3qX4waLG5M43q7Wgc/MbQITxOUSQv7c7ugFFDzQGBzZswY6786m86gpE
Ibb3OhjZnzcvQAaRHhdlQWIMm2nrAgMBAAEwDQYJKoZIhvcNAQEFBQADgYEAtCu4
nUhVVxYUntneD9+h8Mg9q6q+auNKyExzyLwaxlAoo7TJHidbtS4J5iNmZgXL0Fkb
FFBjvSfpJIlJ00zbhNYS5f6GuoEDmFJl0ZxBHjJnyp378OD8uTs7fLvjx79LjSTb
NYiytVbZPQUQ5Yaxu2jXnimvw3rrszlaEXAMPLE"""

AWS_ID_PKCS7_CURL_STATS = """
 % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                  Dload  Upload   Total   Spent    Left  Speed

   0     0    0     0    0     0      0      0 --:--:-- --:--:-- --:--:--     0
 100  1126  100  1126    0     0  1374k      0 --:--:-- --:--:-- --:--:-- 1099k
MIICiTCCAfICCQD6m7oRw0uXOjANBgkqhkiG9w0BAQUFADCBiDELMAkGA1UEBhMC
VVMxCzAJBgNVBAgTAldBMRAwDgYDVQQHEwdTZWF0dGxlMQ8wDQYDVQQKEwZBbWF6
b24xFDASBgNVBAsTC0lBTSBDb25zb2xlMRIwEAYDVQQDEwlUZXN0Q2lsYWMxHzAd
BgkqhkiG9w0BCQEWEG5vb25lQGFtYXpvbi5jb20wHhcNMTEwNDI1MjA0NTIxWhcN
MTIwNDI0MjA0NTIxWjCBiDELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAldBMRAwDgYD
VQQHEwdTZWF0dGxlMQ8wDQYDVQQKEwZBbWF6b24xFDASBgNVBAsTC0lBTSBDb25z
b2xlMRIwEAYDVQQDEwlUZXN0Q2lsYWMxHzAdBgkqhkiG9w0BCQEWEG5vb25lQGFt
YXpvbi5jb20wgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBAMaK0dn+a4GmWIWJ
21uUSfwfEvySWtC2XADZ4nB+BLYgVIk60CpiwsZ3G93vUEIO3IyNoH/f0wYK8m9T
rDHudUZg3qX4waLG5M43q7Wgc/MbQITxOUSQv7c7ugFFDzQGBzZswY6786m86gpE
Ibb3OhjZnzcvQAaRHhdlQWIMm2nrAgMBAAEwDQYJKoZIhvcNAQEFBQADgYEAtCu4
nUhVVxYUntneD9+h8Mg9q6q+auNKyExzyLwaxlAoo7TJHidbtS4J5iNmZgXL0Fkb
FFBjvSfpJIlJ00zbhNYS5f6GuoEDmFJl0ZxBHjJnyp378OD8uTs7fLvjx79LjSTb
NYiytVbZPQUQ5Yaxu2jXnimvw3rrszlaEXAMPLE"""


def test_aws_instance_id_doc():
    with pytest.raises(SkipException):
        AWSInstanceIdDoc(context_wrap(AWS_CURL_ERROR))

    with pytest.raises(SkipException):
        AWSInstanceIdDoc(context_wrap(AWS_NO_DOC))

    with pytest.raises(ParseException) as pe:
        AWSInstanceIdDoc(context_wrap(AWS_ID_DOC_ERROR))
        assert 'Failed to parse json' in str(pe)

    doc = AWSInstanceIdDoc(context_wrap(AWS_ID_DOC))
    assert doc is not None
    assert doc == {
        "devpayProductCodes": None,
        "marketplaceProductCodes": ["1abc2defghijklm3nopqrs4tu", ],
        "availabilityZone": "us-west-2b",
        "privateIp": "10.158.112.84",
        "version": "2017-09-30",
        "instanceId": "i-1234567890abcdef0",
        "billingProducts": ["bp-6ba54002", ],
        "instanceType": "t2.micro",
        "accountId": "123456789012",
        "imageId": "ami-5fb8c835",
        "pendingTime": "2016-11-19T16:32:11Z",
        "architecture": "x86_64",
        "kernelId": None,
        "ramdiskId": None,
        "region": "us-west-2"
    }
    assert "whatchamacallit" not in doc

    doc = AWSInstanceIdDoc(context_wrap(AWS_ID_DOC_CURL_STATS))
    assert doc is not None
    assert doc == {
        "devpayProductCodes": None,
        "marketplaceProductCodes": ["1abc2defghijklm3nopqrs4tu", ],
        "availabilityZone": "us-west-2b",
        "privateIp": "10.158.112.84",
        "version": "2017-09-30",
        "instanceId": "i-1234567890abcdef0",
        "billingProducts": ["bp-6ba54002", ],
        "instanceType": "t2.micro",
        "accountId": "123456789012",
        "imageId": "ami-5fb8c835",
        "pendingTime": "2016-11-19T16:32:11Z",
        "architecture": "x86_64",
        "kernelId": None,
        "ramdiskId": None,
        "region": "us-west-2"
    }
    assert "whatchamacallit" not in doc


def test_aws_instance_id_pkcs7():
    with pytest.raises(SkipException):
        AWSInstanceIdDoc(context_wrap(AWS_CURL_ERROR))

    with pytest.raises(SkipException):
        AWSInstanceIdDoc(context_wrap(AWS_NO_DOC))

    pkcs7 = AWSInstanceIdPkcs7(context_wrap(AWS_ID_PKCS7))
    assert pkcs7 is not None
    assert pkcs7.signature == """
-----BEGIN PKCS7-----
MIICiTCCAfICCQD6m7oRw0uXOjANBgkqhkiG9w0BAQUFADCBiDELMAkGA1UEBhMC
VVMxCzAJBgNVBAgTAldBMRAwDgYDVQQHEwdTZWF0dGxlMQ8wDQYDVQQKEwZBbWF6
b24xFDASBgNVBAsTC0lBTSBDb25zb2xlMRIwEAYDVQQDEwlUZXN0Q2lsYWMxHzAd
BgkqhkiG9w0BCQEWEG5vb25lQGFtYXpvbi5jb20wHhcNMTEwNDI1MjA0NTIxWhcN
MTIwNDI0MjA0NTIxWjCBiDELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAldBMRAwDgYD
VQQHEwdTZWF0dGxlMQ8wDQYDVQQKEwZBbWF6b24xFDASBgNVBAsTC0lBTSBDb25z
b2xlMRIwEAYDVQQDEwlUZXN0Q2lsYWMxHzAdBgkqhkiG9w0BCQEWEG5vb25lQGFt
YXpvbi5jb20wgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBAMaK0dn+a4GmWIWJ
21uUSfwfEvySWtC2XADZ4nB+BLYgVIk60CpiwsZ3G93vUEIO3IyNoH/f0wYK8m9T
rDHudUZg3qX4waLG5M43q7Wgc/MbQITxOUSQv7c7ugFFDzQGBzZswY6786m86gpE
Ibb3OhjZnzcvQAaRHhdlQWIMm2nrAgMBAAEwDQYJKoZIhvcNAQEFBQADgYEAtCu4
nUhVVxYUntneD9+h8Mg9q6q+auNKyExzyLwaxlAoo7TJHidbtS4J5iNmZgXL0Fkb
FFBjvSfpJIlJ00zbhNYS5f6GuoEDmFJl0ZxBHjJnyp378OD8uTs7fLvjx79LjSTb
NYiytVbZPQUQ5Yaxu2jXnimvw3rrszlaEXAMPLE
-----END PKCS7-----""".strip()

    pkcs7 = AWSInstanceIdPkcs7(context_wrap(AWS_ID_PKCS7_CURL_STATS))
    assert pkcs7 is not None
    assert pkcs7.signature == """
-----BEGIN PKCS7-----
MIICiTCCAfICCQD6m7oRw0uXOjANBgkqhkiG9w0BAQUFADCBiDELMAkGA1UEBhMC
VVMxCzAJBgNVBAgTAldBMRAwDgYDVQQHEwdTZWF0dGxlMQ8wDQYDVQQKEwZBbWF6
b24xFDASBgNVBAsTC0lBTSBDb25zb2xlMRIwEAYDVQQDEwlUZXN0Q2lsYWMxHzAd
BgkqhkiG9w0BCQEWEG5vb25lQGFtYXpvbi5jb20wHhcNMTEwNDI1MjA0NTIxWhcN
MTIwNDI0MjA0NTIxWjCBiDELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAldBMRAwDgYD
VQQHEwdTZWF0dGxlMQ8wDQYDVQQKEwZBbWF6b24xFDASBgNVBAsTC0lBTSBDb25z
b2xlMRIwEAYDVQQDEwlUZXN0Q2lsYWMxHzAdBgkqhkiG9w0BCQEWEG5vb25lQGFt
YXpvbi5jb20wgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBAMaK0dn+a4GmWIWJ
21uUSfwfEvySWtC2XADZ4nB+BLYgVIk60CpiwsZ3G93vUEIO3IyNoH/f0wYK8m9T
rDHudUZg3qX4waLG5M43q7Wgc/MbQITxOUSQv7c7ugFFDzQGBzZswY6786m86gpE
Ibb3OhjZnzcvQAaRHhdlQWIMm2nrAgMBAAEwDQYJKoZIhvcNAQEFBQADgYEAtCu4
nUhVVxYUntneD9+h8Mg9q6q+auNKyExzyLwaxlAoo7TJHidbtS4J5iNmZgXL0Fkb
FFBjvSfpJIlJ00zbhNYS5f6GuoEDmFJl0ZxBHjJnyp378OD8uTs7fLvjx79LjSTb
NYiytVbZPQUQ5Yaxu2jXnimvw3rrszlaEXAMPLE
-----END PKCS7-----""".strip()


def test_doc_examples():
    env = {
        'aws_id_doc': AWSInstanceIdDoc(context_wrap(AWS_ID_DOC)),
        'aws_id_sig': AWSInstanceIdPkcs7(context_wrap(AWS_ID_PKCS7)),
    }
    failed, total = doctest.testmod(aws_instance_id, globs=env)
    assert failed == 0



from insights.parsers.rc_local import RcLocal
from insights.tests import context_wrap

RC_LOCAL_DATA = """
#!/bin/sh
#
# This script will be executed *after* all the other init scripts.
# You can put your own initialization stuff in here if you don't
# want to do the full Sys V style init stuff.

touch /var/lock/subsys/local
echo never > /sys/kernel/mm/redhat_transparent_hugepage/enabled
""".strip()


def test_rc_local():
    rc_local = RcLocal(context_wrap(RC_LOCAL_DATA))
    assert len(rc_local.data) == 2
    assert rc_local.data[0] == 'touch /var/lock/subsys/local'
    assert rc_local.get('kernel') == ['echo never > /sys/kernel/mm/redhat_transparent_hugepage/enabled']



import doctest
from datetime import datetime

from insights.parsers import libvirtd_log
from insights.tests import context_wrap


LIBVIRTD_LOG = """
2013-10-23 17:32:19.909+0000: 14069: debug : do_open:1174 : trying driver 0 (Test) ...
2013-10-23 17:32:19.909+0000: 14069: debug : do_open:1180 : driver 0 Test returned DECLINED
2013-10-23 17:32:19.909+0000: 14069: debug : do_open:1174 : trying driver 1 (ESX) ...
2013-10-23 17:32:19.909+0000: 14069: debug : do_open:1180 : driver 1 ESX returned DECLINED
2013-10-23 17:32:19.909+0000: 14069: debug : do_open:1174 : trying driver 2 (remote) ...
2013-10-23 17:32:19.957+0000: 14069: error : virNetTLSContextCheckCertDN:418 : Certificate [session] owner does not match the hostname AA.BB.CC.DD <============= IP Address
2013-10-23 17:32:19.957+0000: 14069: warning : virNetTLSContextCheckCertificate:1102 : Certificate check failed Certificate [session] owner does not match the hostname AA.BB.CC.DD
2013-10-23 17:32:19.957+0000: 14069: error : virNetTLSContextCheckCertificate:1105 : authentication failed: Failed to verify peer's certificate
""".strip()

INSTANCE_001 = """
2019-06-04 05:33:22.280743Z qemu-kvm: -vnc 10.xxx.xxx.xxx:0: Failed to start VNC server: Failed to bind socket: Cannot assign requested address
2019-06-04 05:33:2.285+0000: shutting down
""".strip()


INSTANCE_002 = """
2019-04-26 06:55:20.388+0000: starting up libvirt version: 2.0.0, package: 10.el7_3.9 (Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>, 2017-05-04-06:48:37, x86-034.build.example.com), qemu version: 2.6.0 (qemu-kvm-rhev-2.6.0-28.el7_3.9), hostname: f1.example.com
2019-04-26 06:55:21.280743Z qemu-kvm: -vnc 10.xxx.xxx.xxx:0: Failed to start VNC server: Failed to bind socket: Cannot assign requested address
""".strip()


def test_libvirtd_log():
    log = libvirtd_log.LibVirtdLog(context_wrap(LIBVIRTD_LOG))
    assert "Certificate check failed Certificate" in log

    assert log.get("authentication failed: Failed to verify peer's certificate")[0]['raw_message'] == \
            "2013-10-23 17:32:19.957+0000: 14069: error : virNetTLSContextCheckCertificate:1105 : authentication failed: Failed to verify peer's certificate"
    # Includes continuation of URI component expansion
    assert len(list(log.get_after(datetime(2013, 10, 23, 17, 32, 19)))) == 8


def test_libvirtd_qemu_log():
    log = libvirtd_log.LibVirtdQemuLog(context_wrap(INSTANCE_001, path="/var/log/libvirt/qemu/bb912729-fa51-443b-bac6-bf4c795f082d.log"))
    assert "shutting down" in log

    log = libvirtd_log.LibVirtdQemuLog(context_wrap(INSTANCE_002, path="/var/log/libvirt/qemu/bb912729-fa51-443b-bac6-bf4c795f081d.log"))
    assert log.file_name == 'bb912729-fa51-443b-bac6-bf4c795f081d.log'
    assert len(list(log.get_after(datetime(2019, 4, 26, 6, 55, 20)))) == 2


def test_documentation():
    failed_count, tests = doctest.testmod(
        libvirtd_log,
        globs={'libvirtd_log': libvirtd_log.LibVirtdLog(context_wrap(LIBVIRTD_LOG)),
               'libvirtd_qemu_log': libvirtd_log.LibVirtdQemuLog(context_wrap(INSTANCE_001, path="/var/log/libvirt/qemu/bb912729-fa51-443b-bac6-bf4c795f081d.log"))}
    )
    assert failed_count == 0



"""
The boolean module allows delayed evaluation of boolean expressions. You wrap
predicates in objects that have overloaded operators so they can be connected
symbolically to express ``and``, ``or``, and ``not``. This is useful if you
want to build up a complicated predicate and pass it to something else for
evaluation, in particular :py:class:`insights.parsr.query.Entry` instances.

    .. code-block:: python

        def is_even(n):
            return (n % 2) == 0

        def is_positive(n):
            return n > 0

        even_and_positive = pred(is_even) & pred(is_positive)

        even_and_positive(6) == True
        even_and_positive(-2) == False
        even_and_positive(3) == False

You can also convert two parameter functions to which you want to partially
apply an argument. The arguments partially applied will be those *after* the
first argument. The first argument is the value the function should evaluate
when it's fully applied.

    .. code-block:: python

        import operator
        lt = pred2(operator.lt)  # operator.lt is lt(a, b) == (a < b)
        gt = pred2(operator.gt)  # operator.gt is gt(a, b) == (a > b)

        gt_five = gt(5)  # creates a function of one argument that when called
                         # returns operator.gt(x, 5)

        lt_ten = lt(10)  # creates a function of one argument that when called
                         # returns operator.lt(x, 5)

        gt_five_and_lt_10 = gt(5) & lt(10)

"""


class Boolean(object):
    def __and__(self, other):
        return All(self, other)

    def __or__(self, other):
        return Any(self, other)

    def __invert__(self):
        return Not(self)

    def test(self, value):
        return True

    def __call__(self, value):
        return self.test(value)


class TRUE(Boolean):
    pass


class FALSE(Boolean):
    def test(self, value):
        return False


class Any(Boolean):
    def __init__(self, *exprs):
        self.exprs = list(exprs)

    def test(self, value):
        return any(q.test(value) for q in self.exprs)


class All(Boolean):
    def __init__(self, *exprs):
        self.exprs = list(exprs)

    def test(self, value):
        return all(q.test(value) for q in self.exprs)


class Not(Boolean):
    def __init__(self, query):
        self.query = query

    def test(self, value):
        return not self.query.test(value)


class Predicate(Boolean):
    def __init__(self, func, *args):
        self.func = func
        self.args = args

    def test(self, value):
        try:
            return self.func(value, *self.args)
        except Exception:
            return False


class CaselessPredicate(Predicate):
    def test(self, lhs):
        if isinstance(lhs, str):
            return super(CaselessPredicate, self).test(lhs.lower())
        return super(CaselessPredicate, self).test(lhs)


def pred(func, ignore_case=False):
    if ignore_case:
        return CaselessPredicate(func)
    return Predicate(func)


def pred2(func, ignore_case=False):
    def inner(val):
        if ignore_case:
            return CaselessPredicate(func, val.lower())
        return Predicate(func, val)
    return inner


Or = Any
And = All
TRUE = TRUE()
FALSE = FALSE()



"""
NeutronL3AgentIni - file ``/etc/neutron/l3_agent.ini``
======================================================

The ``NeutronL3AgentIni`` class parses the l3_agent configuration file.
See the ``IniConfigFile`` class for more usage information.
"""

from insights import add_filter, parser, IniConfigFile
from insights.specs import Specs

add_filter(Specs.neutron_l3_agent_ini, ["["])


@parser(Specs.neutron_l3_agent_ini)
class NeutronL3AgentIni(IniConfigFile):
    """
    Parse the ``/etc/neutron/l3_agent.ini`` configuration file.

    Sample configuration::

         [DEFAULT]

         #
         # From neutron.base.agent
         #

         # Name of Open vSwitch bridge to use (string value)
         ovs_integration_bridge = br-int

         # Uses veth for an OVS interface or not. Support kernels with limited namespace
         # support (e.g. RHEL 6.5) so long as ovs_use_veth is set to True. (boolean
         # value)
         ovs_use_veth = false

         # The driver used to manage the virtual interface. (string value)
         #interface_driver = <None>
         interface_driver = neutron.agent.linux.interface.OVSInterfaceDriver

         # Timeout in seconds for ovs-vsctl commands. If the timeout expires, ovs
         # commands will fail with ALARMCLOCK error. (integer value)
         ovs_vsctl_timeout = 10

         #
         # From neutron.l3.agent
         #

         # The working mode for the agent. Allowed modes are: 'legacy' - this preserves
         # the existing behavior where the L3 agent is deployed on a centralized
         # networking node to provide L3 services like DNAT, and SNAT. Use this mode if
         # you do not want to adopt DVR. 'dvr' - this mode enables DVR functionality and
         # must be used for an L3 agent that runs on a compute host. 'dvr_snat' - this
         # enables centralized SNAT support in conjunction with DVR.  This mode must be
         # used for an L3 agent running on a centralized node (or in single-host
         # deployments, e.g. devstack) (string value)
         # Allowed values: dvr, dvr_snat, legacy
         agent_mode = dvr

         # TCP Port used by Neutron metadata namespace proxy. (port value)
         # Minimum value: 0
         # Maximum value: 65535
         metadata_port = 9697

         # Send this many gratuitous ARPs for HA setup, if less than or equal to 0, the
         # feature is disabled (integer value)
         #send_arp_for_ha = 3

         # Allow running metadata proxy. (boolean value)
         enable_metadata_proxy = true

         # DEPRECATED: Name of bridge used for external network traffic. When this
         # parameter is set, the L3 agent will plug an interface directly into an
         # external bridge which will not allow any wiring by the L2 agent. Using this
         # will result in incorrect port statuses. This option is deprecated and will be
         # removed in Ocata. (string value)
         # This option is deprecated for removal.
         # Its value may be silently ignored in the future.
         external_network_bridge =

         #
         # From oslo.log
         #

         # If set to true, the logging level will be set to DEBUG instead of the default
         # INFO level. (boolean value)
         # Note: This option can be changed without restarting.
         debug = False

         # Defines the format string for %%(asctime)s in log records. Default:
         # %(default)s . This option is ignored if log_config_append is set. (string
         # value)
         log_date_format = %Y-%m-%d %H:%M:%S

         [AGENT]

         #
         # From neutron.base.agent
         #

         # Seconds between nodes reporting state to server; should be less than
         # agent_down_time, best if it is half or less than agent_down_time. (floating
         # point value)
         report_interval = 30

         # Log agent heartbeats (boolean value)
         log_agent_heartbeats = false

         # Availability zone of this node (string value)
         availability_zone = nova


    Examples:

        >>> l3_agent_ini.has_option("AGENT", "log_agent_heartbeats")
        True
        >>> l3_agent_ini.get("DEFAULT", "agent_mode") == "dvr"
        True
        >>> l3_agent_ini.getint("DEFAULT", "metadata_port")
        9697

    """
    pass
